### GramChar.R
###------------------------------------------------------------------------
### What: Gram-Charlier approximation of the marginal posterior density
### $Id$
### Time-stamp: <2007-04-14 14:19:20 ggorjan>
### Changes by Gabor Meszaros, 01. March 2011
###------------------------------------------------------------------------

GramChar <- function(mean, sd, skew, variable="x" ,plot=TRUE, ...)
{
  ## --- Setup ---

  mL <- length(mean)
  dL <- length(sd)
  sL <- length(skew)
  m <- max(mL, dL, sL)

  ret <- list()
  ret$mean <- vector(length=m)
  suppressWarnings(ret$mean[1:m] <- mean)
  ret$sd <- vector(length=m)
  suppressWarnings(ret$sd[1:m] <- sd)
  ret$skew <- vector(length=m)
  suppressWarnings(ret$skew[1:m] <- skew)
  ret$z <- seq(from=-5, to=5, by=0.1)
  ret$x <- matrix(nrow=length(ret$z), ncol=m)
  ret$y <- ret$x

  ## --- Calculate ---

  for(i in 1:m) {
    ret$x[, i] <- ret$mean[i] + ret$z * ret$sd[i]
    ret$y[, i] <- dnorm(ret$z, mean=0, sd=1) *
                  (1 + ret$skew[i] * (ret$z^3 - 3*ret$z)/6)
  }

  ## --- Plot ---
#
  if(plot) {
    plot(x=ret$x, y=ret$y, xlim=c(0, max(ret$x)), type="l", col="blue",
            ylab="Density", xlab=variable, ...)
    title("Gram-Charlier approximation of the marginal posterior density")
  }

  ## --- Return ---

  invisible(ret)
}

###------------------------------------------------------------------------
### GramChar.R ends here
