\name{coxWei4R}
\alias{coxWei4R}
\title{
coxWei4R
}
\description{
This function is an R interface for Cox and Weibull programs of the Survival Kit. It creates a parameter file with a specific structure and calls the pre compiled executable file.
}
\usage{
coxWei4R(useR = TRUE, nrecmax = 0, ndimax = 0, nefmax = 0, 
nstramax = 0, ntimmax = 0, nstimax = 0, nlevout = 0, 
npgauss = 0, niter_gauss = 0, nvec_bf = 0, no_logarithm = -1, 
no_hessian = FALSE, nze_hessian = 0, with_mgd = FALSE, 
logfile = "logfile.txt", files = c("", "", "", "", "", ""), 
title = "", ite_quasi = -1, strata = "", origin = "", 
rho_fixed = 0, model = c("", "", ""), coefficient = "", 
statistics = FALSE, only_statistics = FALSE, random = "", 
integrate_out = "", animal_solution = "", std_error = TRUE, 
dense_hessian = FALSE, force_positive = FALSE, 
baseline = FALSE, kaplan = FALSE, moments = FALSE, 
survivalFiles = "", survivalOptions = "", residualFiles = "", 
constraintsOptions = "", convergence = 0, 
store_solutions = FALSE, read_old_solutions = FALSE, 
storage = "IN_CORE", test = "", correlation = c("", "", ""), 
program = "", UNIXlike = FALSE, verbose = FALSE, 
graphics = TRUE, residual = FALSE, wdNew = "", 
wdSetBack = TRUE)
}
\arguments{
  \item{useR}{
If set to TRUE additional output files are created to use the grapical capacities of R.
}
  \item{nrecmax}{
Sets the maximum number of elementary records in recoded data file. With time-dependent variables this number may be much larger than the number of records in your original data file (NRECMAX statement of the Kit).
}
  \item{ndimax}{
Sets the maximum total number of levels of effects in the model, size of the vector of solutions (NDIMAX statement of the Kit).
}
  \item{nefmax}{
Sets the maximum number of covariates. These are both discrete covariates and continuous covariates. Time dependent covariates have to be counted twice, because the states before and after change occupy two positions on the recoded data file.
}
  \item{nstramax}{
ets the maximum number of strata, i.e. levels of the strata variable defined in the strata statement (NSTRAMAX statement of the Kit).
}
  \item{ntimmax}{
ets the largest possible value of the (dependent) time variable (NTIMMAX statement of the Kit).
}
  \item{nstimax}{
Sets the maximum number of distinct times or quantiles that can be defined in the survival statement options (NSTIMAX  statement of the Kit).
}
  \item{nlevout}{
Sets the maximum number of levels for the random effect which is integrated out (NLEVOUT statement of the Kit).
}
  \item{npgauss}{
Defines the number of points used during Gauss-Hermite integration (has to be between 3 and 5), when moments statement is used (NPGAUSS statement of the Kit).
}
  \item{niter_gauss}{
Gives the number of iterations in the Gauss-Hermite integration process, when moments statement is used (NITER_GAUSS statement of the Kit).
}
  \item{nvec_bf}{
Defines the number of vectors used to store the approximation of the Hessian (the rank of approximation of Hessian matrix). Values between 3 and 20 might be used (NVEC_BF statement of the Kit).
}
  \item{no_logarithm}{
Defines the form of rho used in Weibull. If set to 1 rho is used, if set to 0 log(rho) is used (NO_LOGARITHM statement of the Kit).
}
  \item{no_hessian}{
If set to TRUE the Hessian matrix is omitted  from the computations, otherwise it is stored by default (NO_HESSIAN statement of the Kit).
}
  \item{nze_hessian}{
Declares the number of nonzero elements in the Hessian matrix (NZE_HESSIAN statement of the Kit).
}
  \item{with_mgd}{
If TRUE, it indicates if the maternal granddam was included into the evaluation (WITH_MGD statement of the Kit).
}
  \item{logfile}{
Specifies the name of the log file where computation details from the Survival Kit will be printed. If a missing filename is specified ("") with verbose=TRUE, the output will be printed to screen.
}
  \item{files}{
A character vector of up to 6 elements holding the names of input and output files for Cox and Weibull midels.

1. \emph{file1} is the name of the recoded data file (\emph{file2} of FILES Statement
in PREPARE). This file must exist in your directory.

2. \emph{file2} is the name of the file containing the original and new 
codes (after internal recoding) for each CLASS variable.
(\emph{file3} of FILES Statement in PREPARE).
The file name is required even when no CLASS variables were defined in PREPARE.
In this case it will be an empty file.

3. \emph{file3} is the name of the output file where the results of the
analysis are listed. 

4. \emph{file4} is the (recoded) pedigree file \emph{file8} of the
PEDIGREE Statement of PREPARE. This file is not obligatory and will only
be needed for analyses where the additive genetic relationship structure
between individuals is considered.

5. \emph{file5} is only needed when for very large applications, solutions from
a previous run are to be read in because the READ\_OLD\_SOLUTIONS statement is used.

6. \emph{file6} is only needed when for very large applications, solutions are
 to be stored for a subsequent run with the STORE\_SOLUTIONS statement.

Files 1, 2 must exist in your directory, files 4 and 5 must exist when
subsequent statements make use of them; file 3 and 6 will be new files
(Note: existing files with the same name are overwritten without warning).
In situations where files 5 or 6 are required but not files 4 or 4 and 5,
dummy names have to be stated for the latter ones.

}
  \item{title}{
The title of the analysis, to be written at the beginning of the output files (TITLE statement of the Kit).
}
  \item{ite_quasi}{
Specifies the number of quasi-newton iterations before swithching to full Newton steps (ITE_QUASI statement of the Kit). In case of 0 iterations the program will start with the full Newton. 
}
  \item{strata}{
Specifies the name of the stratification variable. Stratification is an extension of the proportional hazards model. With stratification, the assumption of  the proportionality of hazards is restricted to individuals within subgroups of the population, where grouping is defined by the variable indicated in this statement (STRATA statement of the Kit). Only one variable can be used as strata variable.
}
  \item{origin}{
If an integer constant is specified, the program takes this as a starting time point for the evaluation. The value of the constant should be lower than the time of the first event.
In case a class variable is specified here, the program will assume a new starting point for each change of the variable (ORIGIN statement of the Kit). 
}
  \item{rho_fixed}{
Specifies a fixed value for the Weibull parameter, only useable with the Weibull program (RHO_FIXED statement of the Kit).
}
  \item{model}{
Specifies the independent variables affecting the dependent variable described in the time statement of prepare (MODEL statement of the Kit).
}
  \item{coefficient}{
Specifies the effect name and coefficient value. During the computations the specific covariate will always be multiplied by the coefficient value (COEFFICIENT statement of the Kit).
}
  \item{statistics}{
The function argument requests the computation and the printing of a number of elementary statistics related to each level of the class variables specified in the model statement (number of observations, number of observed failures, age at failure, average value of continuous covariates for all observations and for the uncensored ones) (STATISCTICS statement of the Kit).
}
  \item{only_statistics}{
If set to TRUE, only the general statistics will be computed and the progam stops.
}
  \item{random}{
random=("variable <ESTIMATE <MOMENTS>> distribution <rules> parameter<s>  <repeat previous sequence for next variable<s>>")

The RANDOM statement gives information on \emph{variables} to be
treated as random. The distribution parameters  of the random
covariates are either assumed to be known or they may be estimated
(optional parameter ESTIMATE).

The distribution allows to specify the distribution 
 that the random variable is
assumed to follow. The user may choose from 3 alternatives:

\emph{LOGGAMMA}: the levels of the random effect follow a log-gamma
distribution. This is identical with assuming a gamma frailty term.
The frailty term, say w, is defined as a multiplicative term to
the usual hazard function with fixed effects only. 

\emph{NORMAL}: the levels of the random effect are independently
normally distributed.
 This assumption is not so common in survival
analysis but for rather large gamma parameters gamma>10, the log-gamma and
normal are similar and the normal distribution is needed to make
the step to the multivariate normal distribution described
next.
\emph{MULTINORMAL rules}: the levels of the random effect
follow a multivariate normal
distribution with covariances between levels being induced by genetic
relationships. Two types of relationships are allowed and may be stated via
the \emph{rules} parameter: \emph{USUAL_RULES}  are the relationships
under an animal model; \emph{MGS_RULES} relate to a sire model or a sire-maternal
grandsire model, accounting for male relationships. 
When MULTINORMAL is used, information about the covariances between individuals
has to be provided via a pedigree file.

\emph{parameter} is the distribution parameter (gamma parameter with the
log-gamma distribution and the variance with normal or multivariate normal
distributions). It may be preset, in which case one value of the parameter
has to be provided or it may be estimated (see ESTIMATE option below), in
which case three values have to be given (see below).

\emph{ESTIMATE}:  when stated after the variable name,
the distribution parameter is estimated as the mode of  its marginal posterior density
which is approximated by Laplacian integration.
The \emph{parameter} value
is replaced by three values: the first two values are the bounds
of the interval to be searched, the third value gives the final tolerance
(accuracy). Using the ESTIMATE option, the parameter of only one random
variable may be estimated at one time. If the estimated value is at one
of the bounds prespecified, a warning message is printed and the analysis should be repeated changing
the values of the bounds. (Example: you stated parameters 0.01 0.1 0.001
as lower bound, upper bound and tolerance, which means that the program must look for the 
mode of the marginal posterior density in the interval ]0.01, 0.1[ and the searching process is
stopped when the current interval for the estimate is smaller than 0.001.
If the program output tells
you "the mode is between 0.0100 and 0.0109 and the best value is
0.0104", you need to reset the bounds, e.g., to  0.005 and 0.011 and start again).
The STORE_SOLUTIONS and READ_OLD_SOLUTIONS 
statements described below may be used to avoid starting from scratch
again.
When the INTEGRATE_OUT  JOINT_MODE  statement is used (only with a Weibull model and for a log-gamma random effect), the ESTIMATE statement should not be used (for the log-gamma random effect): the gamma parameter is then estimated jointly with the other effects after exact algebraic integration of the log-gamma random effect. 

\emph{MOMENTS}: this option may be used together with ESTIMATE to
compute estimated mean, standard deviation and skewness of the
marginal persterior value of the distributional parameter. The computation is
based on an iterated Gauss-Hermite quadrature of the approximate marginal
posterior densities. The number of points used for the integration is NPGAUSS, changable in the parameter file (see NPGAUSS statement).

 When the amount of information available for the estimation
is limited, unpredictable results (if any) may be obtained, although the mode
of the distribution may have been successfully computed.
}
  \item{integrate_out}{
 integrate_out=("<JOINT_MODE> <BACK_SOLVE> variable <WITHIN variable2>")

 This option is only available in the WEIBULL program (the COX program prints an error message and stops) and is \emph{not} applicable to a grouped data model. 
When a random effect is assumed to follow a log-gamma distribution in a Weibull model, it is possible to algebraically integrate it out from the joint posterior density. 
This technique decreases the number of parameters to estimate (sometimes drastically). Algebraic integration is equivalent here to absorption of a group of equations in systems of linear equations. The consequences are similar: a smaller system (but usually much less sparse) and no direct availability of the estimates of the effects integrated out (other than the gamma parameter).

 The INTEGRATE_OUT statement can be used alone, i.e., only followed by the name of the random variable to integrate out (must appear in the RANDOM statement too).
Then, the gamma parameter of the log-gamma distribution is assumed to be known. For example,
  model=c("effect1","effect2","effect3")
  random="LOGGAMMA effect1 10.0")
  integrate_out="effect1"
will lead to the integration of the log-gamma distributed random variable \emph{effect1} and the estimation of effects \emph{effect2} and \emph{effect3} and of the Weibull parameters, assuming a value of 10.0 for the gamma parameter of \emph{effect1}.

 If the  INTEGRATE_OUT statement  is used with the  JOINT_MODE option, (as   INTEGRATE_OUT JOINT_MODE effect1; in the example above) then the gamma parameter is estimated jointly with the other effects as the mode of the marginal posterior distribution of the gamma parameter, effect2,  effect3 and the Weibull parameters. The value 10.0 specified in the RANDOM statement is just used as a starting value in the optimisation subroutine.
The advantage of this approach is that it performs \emph{exactly} the marginalisation of the random effect, which otherwise is done only approximately via Laplacian integration. Note, however, that the fixed and other random effects as well as the Weibull parameters still have to be integrated out if one wants the \emph{full} marginal posterior density for the gamma parameter of the log-gamma distribution).
 The combined use of RANDOM ESTIMATE \dots and INTEGRATE_OUT JOINT_MODE \dots offers an easy solution to the joint estimation of the distribution parameters of two random variables.
}
  \item{animal_solution}{
Filename for the results of approximate animal model. The random effect should be integrated out in the model via integrate_out statement, the usage of back_solve statement is needed.
}
  \item{std_error}{
If set to TRUE asymptotic standard errors of estimates are computed and printed in the main results file (STD_ERROR statement of the Kit).
}
  \item{dense_hessian}{
If set to TRUE a dense Hessian matrix is used (DENSE_HESSIAN statement of the Kit). 
The full storage of the square Hessian matrix is the only option when the COX program is used (the DENSE_HESSIAN statement is not required). The default option for the WEIBULL program is the sparse storage of the Hessian, unless the DENSE_HESSIAN statement is used. In cases where the memory is not limiting or when a log-gamma random effect is algebraically integrated out (leading to smaller but denser Hessian), the DENSE_HESSIAN  option may save substantial computing time.
}
  \item{force_positive}{
In certain cases a "matrix is not positive definite" message might appear, which can be bypassed setting the force_positive function argument to TRUE (FORCE_POSITIVE statement of the Kit).
}
  \item{baseline}{
When set to TRUE, the statement specifies that the baseline hazard, the baseline cumulative hazard and 
the baseline survivor functions should be computed and printed, in the COX program (BASELINE statement of the Kit).
These values are computed at each distinct value of the TIME variable.
In stratified analyses (use of the STRATA statement), separate 
baseline functions are calculated for strata. With the WEIBULL program, the computation of the baseline Weibull parameters is always implicit: the BASELINE statement is ignored.
}
  \item{kaplan}{
When set to TRUE, the statement specifies that the product-limit (=Kaplan-Meier) estimate 
of the survivor function should be computed and printed (KAPLAN statement of the Kit).. The Kaplan-Meier
estimator is a non-parametric population estimator of the survivor function
that does not take into account any of the effects stated in the MODEL statement.  The KAPLAN statement is permitted only with the COX program (it is ignored otherwise).
}
  \item{moments}{
Setting it to TRUE indicates that the moments statement was used for the random effect and the Gram-Charlier approximations for the distribution of the random effect should be computed and visualized.
}
  \item{survivalFiles}{
Specifies the two filenames to be used for the SURVIVAL keyword of the Kit.
It is very useful to relate the estimated regression coefficients
to a more conventional scale like median survival time or
probability of survival to a certain age.

1. \emph{file1} is the name of the recoded file produced by program PREPARE
on request in the FUTURE statement of PREPARE (called \emph{file2} there).

2. \emph{file2} holds the results invoked by the use of the SURVIVAL statement.
}
  \item{survivalOptions}{
There are several possibilities to compute the survivor function of individuals with certain covariate values.
The following \emph{options} may be used:

\emph{QUANTILES value<s>} 
With this option, times at which specific values (quantiles) of the
survivor curve are reached are calculated and printed for each individual.
 The \emph{values} must lie between 0 and 100 and
will be divided by 100 to produce the value of the survivor function (e.g.,
with a value of 25, S(t)= 0.25). The median survival time can therefore
be requested by the option QUANTILE 50.

\emph{ALL_TIMES}
The estimate of the survivor curve will be computed
at all times (= failure time or change in time-dependent
covariates) indicated for each individual.

\emph{EQUAL_SPACE interval limit} 
The estimate of the survivor curve will be computed
at equally spaced times for each individual.
two figures are needed: time interval and time limit.
For example, SURVIVAL EQUAL_SPACE  5 100  will
compute the value of the survivor curve at time 
5, 10, 15, \dots 95, 100.

\emph{SPECIFIC time<s>} 
The estimate of the survivor curve will be computed
at the specified times indicated for each individual.
}
  \item{residualFiles}{
Character vector holding the two file names to be included into the RESIDUAL keyword of the Kit, for whitch generalized residuals should be computed. 

See also the residual functin argument. 
}
  \item{constraintsOptions}{
For models not of full rank (all models with fixed
discrete class covariates), 
constraints can be imposed upon the effects to be estimated to
get a set of meaningfull, easy to interpret 
estimable effects. The program offers different
\emph{options} for setting those constraints:

\emph{LARGEST}: the program will set to zero the level of each discrete
covariate with the largest number of uncensored failures. This is the
default procedure of handling constraints.

\emph{FIND}: the constraints are found by the program. This will guarantee
the correct number of constraints to produce a full rank Hessian
matrix, so that the tests for the full model as well as for individual
effects are based on the correct degrees of freedom (however, the constraints found do not guarantee an easy interpretation of the results...) Linear dependencies in the Hessian matrix are found by performing a Cholesky decomposition. For very large problems, the storage of this matrix may be a limiting factor (as for the computation of the standard error; see the STD_ERROR statement).

\emph{NONE}: no constraints specified (but implicit ones will appear
if standard errors are to be computed, i.e. a generalized
inverse of the Hessian matrix will be used if needed).

\emph{IMPOSE <CHECK> variable rec_level variable rec_level ...}: the
constraints are supplied by the user. \emph{variable} defines the effect for
which a constraint is to be set (must be a CLASS variable that has
been stated in the MODEL statement). \emph{rec_level} is the recoded
value of the effect that should be constraint.
\emph{Note:} You have to
look into the file that holds original and recoded levels of the CLASS
effects (\emph{file3} defined in the PREPARE
parameter file, codelist.txt by default) to look up what is the
recoded value of the level you actually want to put the constraint on.
To do so, look for the name of the effect of interest. Columns 4 (when there is no interaction), 4 and 5 (when two effects were combined into an interaction) or 4, 5, and 6 (when three effects were combined) display the original code. The last column defines the new code.
By use of the option \emph{check} the program will check whether the
constraints specified by the user make sense.
}
  \item{convergence}{
The \emph{real_number} in the CONVERGENCE_CRITERION
statement of the Survival Kit provides the  termination
point for the optimisation routine used in the likelihood maximisation.
Its default value is 1.D-8, but alternative values may be invoked by using this statement.
\emph{Warning}: an unnecessary strict convergence criterion set by the
user will lead to a warning message by the optimisation routine telling
(amongst others) "LINE SEARCH FAILED", although the results are
printed and usually  valid.
}
  \item{store_solutions}{
The two statements store_solutions and read_old_solutions, invoked by setting their value to TRUE,  are only useful for very large applications (applications running for hours and days). If the STORE_SOLUTIONS statement has been used
to write the final solutions, the READ_OLD_SOLUTIONS statement may be used in the next run to retrieve
those solutions. The model does not need to
be the same in both runs. This is mainly useful when one is fitting a complex model from a simpler one or in connection with the
ESTIMATE option of the RANDOM statement. When the results of the estimation
indicate that the true value of the distributional parameter for the log-gamma
or Normal distributions lies outside the prespecified interval, the
values of the interval may be changed and using READ_OLD_SOLUTIONS
the optimisation may proceed from the point reached already.
}
  \item{read_old_solutions}{
See store_solutions above.
}
  \item{storage}{
The data file may either be read in and held in core (option
\emph{IN_CORE} or while being read for the first time, be written out
on a temporary file in binary mode on disk (option \emph{ON_DISK}) 
for faster access in subsequent readings of the file in cases where
the core memory available is not big enough to hold the whole data file.
The largest number of elementary records possibly stored in core at the same time is specified by the parameter  NRECMAX changable in the parameter text file (see NRECMAX statement).
}
  \item{test}{
Hypothesis testing is generally performed via likelihood ratio tests.
The following types of tests may be requested by the TEST statement:
(\emph{GENERAL}:  test of the full model vs the model with no covariate.
 This is the default option and will be used even
without request.) 
\emph{SEQUENTIAL}: test of the effects included in the model in sequential
order (i.e. depending on the sequence in the MODEL statement). 
\emph{LAST}: likelihood ratio test comparing the full model with models
excluding one effect at a time. This is done for each effect separately.
\emph{EFFECT list of variables}: the same type of test as with LAST 
is performed, but only for effects stated in the list.

\emph{SEQUENTIAL} and \emph{LAST} may be stated together. \emph{EFFECT} may be
stated either alone,  with \emph{SEQUENTIAL}  or with \emph{LAST}. When \emph{EFFECT} is used with
\emph{LAST}, it is redundant. If it is used with \emph{SEQUENTIAL},
 the sequential inclusion of the effects in the model one at a time
will start with the first effect appearing in the MODEL statement which is stated in the list
following  \emph{EFFECT}. For example, 
  model=c("var1","var2","var3","var4","var5","var6"),
  test="SEQUENTIAL EFFECT var4 var5 var6"
 will lead to likelihood ratio tests corresponding to the sequential introduction
of variables var4, var5, var6.
}
  \item{correlation}{
A character string holding the names of two correlated random effects and the value of the correlation coefficient, or the command to estimate its value and a starting value for the estimate.
}
  \item{program}{
Indicates which compiled executable file should be used ("cox" or "weibull") after data preparation with prepare4R. This is a compulsory parameter.
}
  \item{UNIXlike}{
If set to TRUE, the executable file name is preceeded by "./" and runs on UNIX-like systems. The executable files for the Survival Kit has to be compiled for that particular system. The executable compiled for Windows will not run on Linux, and the other way around.
}
  \item{verbose}{
If set to TRUE, the output of the program displayed in the shell or command propt is also printed in R.
}
  \item{graphics}{
If set to TRUE the graphical capabilities of R are used to visualize the results.
}
  \item{residual}{
If set to TRUE, generalized residuals are computed by Survival Kit.
}
  \item{wdNew}{
Sets the R working directory to one specified by the user (for example: "C:/newDirectory").
}
  \item{wdSetBack}{
If TRUE is specified, the R working directory will be set back to the original one, in FALSE is specified, it remains the same as set in \emph{wdNew}.
}
}
\details{
More details regarding Survival Kit keywords can be found in the manual available from the website (see references below).
}
\author{
Vincent Ducrocq, Johann Soelkner, Gabor Meszaros \cr
Maintainer: Gabor Meszaros <gabor.meszaros@boku.ac.at>
}
\references{
The source code, compiled executable files, manual, reference paper and support programs for the Survival Kit can be found at: http://www.nas.boku.ac.at/nuwi/the-survival-kit/ \cr
The preferred citation of the software including the usage of the R interface is: \cr
Meszaros, G., Soelkner, J., Ducrocq, V. (2013): The Survival Kit: Software to analyze survival data including possibly correlated random effects. Computer Methods and Programs in Biomedicine, 110(3): 503-510. DOI:10.1016/j.cmpb.2013.01.010
}
