\name{prepare4R}
\alias{prepare4R}
\title{
prepare4R
}
\description{
This function is an R interface for the data preparation part of the Survival Kit. It creates a parameter file with a specific structure and calls the pre compiled executable file.
}
\usage{
prepare4R(nrecmax = 0, nefmax = 0, ndimax = 0, files = c("","","",""), 
futureFilenames = c("", ""), pedigreeFilenames = c("", ""), 
pedigreeEffect = "", effects = c("", ""), effectType = c("", ""), 
classEffects = c("", ""), timeDepEffects = c(""), 
timeDepEffectsType = c(""), joinCodeEffects = "", 
outputEffects = "", time, censName, censValue, idRec, 
truncate = "", discrete, timeCovName = "", timeCovType = "", 
timeCovValues = "", newEffectConvdt = "", convdtEffects = "", 
newEffectCombine = "", combineEffects = "", formout = "free_format", 
correlation = c("", "", ""), UNIXlike = FALSE, verbose = FALSE, 
survivalOptions, inputPedigree, wdNew = "", wdSetBack = TRUE, program)
}
\arguments{
  \item{nrecmax}{
ets the maximum number of elementary records in recoded data file. With time-dependent variables this number may be much larger than the number of records in your original data file (NRECMAX statement of the Kit).
}
  \item{nefmax}{
Sets the maximum number of covariates. These are both discrete covariates and continuous covariates. Time dependent covariates have to be counted twice, because the states before and after change occupy two positions on the recoded data file.
}
  \item{ndimax}{
Sets the maximum total number of levels of effects in the model, size of the vector of solutions (NDIMAX statement of the Kit).
}
  \item{files}{
A vector consisting from 4 filenames required for the data preparation program. These are set automatically when using the SKit4R overarching program. The files should be in the following order:

1. \emph{file1} is the name of the original data file (must exist in your directory).

2.\emph{file2} is the name of the recoded file produced by the program to be used by subsequent programs COX or WEIBULL. This file may be much larger than the original data file when time dependent covariates are used in the analysis, as so-called elementary record are written spanning the times between any changes in time dependent covariates. 

3. \emph{file3} is the name of a file containing the original and new codes (after internal recoding) for each variable in the class statement. The file name is required even when no class statement is defined. In this case it will be an empty file. 

4. \emph{file4} is the name of a file containing information about the variables in the data set to be used by subsequent COX or WEIBULL. It is one of two parameter files required by those programs. From v6 onwards this file name is set to \emph{varlist.txt} by default, i.e. this name should appear as fourth in the parameter file for PREPARE. 

}
  \item{futureFilenames}{
In the FUTURE statement, you provide information about a set of records, for which you want a printout on predicted values of the survivor curve (quantile values or functional values of the survival function at given times). 

Two filenames should be provided: First is the name of the original file holding the 'future' records (must exist in your directory). The structure of the file must be exactly identical to that of the first file of the FILES statement.\\
The second is the name of the recoded file produced by the program identical in structure to the second file of the FILES statement. It will be used by COX or WEIBULL to produce values related to the corresponding survival functions. Note: if you have a future statement, you also need to have an IDREC statement, as elementary records have to be sorted by identity of the original records. 

See also the survivalOptions function argument.
}

  \item{pedigreeFilenames}{
Character vector consisting from 2 elements: Name of the input pedigree file (must exist in the R working directory) and the name of the recoded pedigree file to be produced by the data preparation program of Surrvival Kit.

See also the inputPedigree function argument.
}
  \item{pedigreeEffect}{
The name of the random effect associated with the relationship matrix (PEDIGREE statment in the Kit). 
}
  \item{effects}{
Character vector of effect names included in the file1 of the files function argument (INPUT statement of the Kit). 
}
  \item{effectType}{
Type specification for each effect. If only integer values are used in the inputData,this function argument could be omitted. In other case use "class" for integer, "continuous" for real, "date6" for date in 6 digit format (ddmmyy) and "date8" for date in 8 digit format (ddmmyyyy).
}
  \item{classEffects}{
Character vector of variable names to be treated as discontinuous, class covariates (CLASS statement of the Kit). Everything else stated in effects function argument is treated as a continuous covariate.
}
  \item{timeDepEffects}{
Character vector of time dependent variable names (TIMEDEP statement of the Kit).
}
  \item{timeDepEffectsType}{
Type specification for each time dependent effect. This parameter should contain type specifications as described in effectType above, or could be omitted in case of only integer values.
}
  \item{joinCodeEffects}{
Character vector which specifies the two variables to be recoded together, as if they pertained to a single list (JOINCODE statement of the Kit).
}
  \item{outputEffects}{
Vector of effects to be included to the internal recoded file. In case of large datasets a considerable amount of space and memory may be saved if only the variables needed for further analysis are included here. If the keyword is omitted, all effects are included into the output data set by default. 
}
  \item{time}{
This could be a single variable name holding the name of an already computed survival time, or a set of 3 variables holding the name of the survival time, the beginning and the end of the observation given as dates. In the latter case the total survival time is computed within the program (TIME statement of the Kit). Only integer time variables are allowed. If the user specifies "life" as the time variable name in effects, the function argument could be omitted.
}
  \item{censName}{
Name of the variable holding the censoring codes for each record. If the user specifies "cens" as the censoring variable name in effects, the function argument could be omitted.
}
  \item{censValue}{
A number that identifies the censored observations in censName, everything else is considered as an uncensored record. If the user specifies 0 as the value denoting censored records, the function argument could be omitted.
}
  \item{idRec}{
Name of the variable holding the unique identification number for each record. If the user specifies "idnum" as the ID variable name in effects, the function argument could be omitted.
}
  \item{truncate}{
In case there are left truncated records in the data set, this variable holds the time of the truncation point, which could be either an integer number or a date. If the variable is coded 0 in the data set, the record is treated as not truncated, independently from data type (TRUNCATE statement of the Kit).
}
  \item{discrete}{
If set to TRUE, it indicates that the time scale is expressed in a few discrete units, therefore the grouped data model is used (DISCRETE statement of the Kit). This is available with the weibull executable file (although it is not a Weibull model!).
}
  \item{timeCovName}{
efines the name of the time dependent variable created by the Survival Kit. This must be a new name not specified previously (in the effects or time statements).
}
  \item{timeCovType}{
Defines the type of the inout values, from which the new variable will be created. It may be either integer ("I4") or a date ("D6" or "D8").
}
  \item{timeCovValues}{
Indicates the list of values of where the newly creatred effet will change in time (TIMECOV statement of the Kit). 
}
  \item{newEffectConvdt}{
Variable name for the calculated difference between two date variables (CONVDT statement of the Kit).
}
  \item{convdtEffects}{
Character vector holding the names of the two effects used to calculate the values of newEffectConvdt.
}
  \item{newEffectCombine}{
The name of the interaction effect to be produced by combination of two existing class effects(COMBINE statement of the Kit). Continuous variables are not allowed. 
}
  \item{combineEffects}{
Character vector holding the names of the two varianbles to be combined to an interaction term.
}
  \item{formout}{
Specification of the output format (FORMOUT statement of the Kit). 
}
  \item{correlation}{
Character vector holding the names of the two correlated random effects. There is no computation done in this step, the names are only indications for the Cox or Weibull program.
}
  \item{UNIXlike}{
If set to TRUE, the executable file name is preceeded by "./" and runs on UNIX-like systems. The executable files for the Survival Kit has to be compiled for that particular system. The executable compiled for Windows will not run on Linux, and the other way around.
}
  \item{verbose}{
If set to TRUE, the output of the program displayed in the shell or command propt is also printed in R.
}
  \item{survivalOptions}{ 
Specify "useFuture" if the FUTURE statement should be used in the data prepaparation step via futureFilenames,or set to missing ("") otherwise. This function argument in specified for the purposes of the SKit4R over arching function.
}
  \item{inputPedigree}{
Specify "usePedigree" if the PEDIGREE statement should be used in the data prepaparation step via pedigreeFilenames,or set to missing ("") otherwise. This function argument in specified for the purposes of the SKit4R over arching function.
}
  \item{wdNew}{
Sets the R working directory to one specified by the user (for example: "C:/newDirectory").
}
  \item{wdSetBack}{
If TRUE is specified, the R working directory will be set back to the original one, in FALSE is specified, it remains the same as set in \emph{wdNew}.
}
  \item{program}{
Specify "prepare" to run the program.  This function argument in specified for the purposes of the SKit4R over arching function. If "only_cox" or "only_weibull" are specified here the data preparation step will not be executed.
}
}
\details{
More details regarding Survival Kit keywords can be found in the manual available from the website (see references below).
}
\author{
Vincent Ducrocq, Johann Soelkner, Gabor Meszaros \cr
Maintainer: Gabor Meszaros <gabor.meszaros@boku.ac.at>
}
\references{
The source code, compiled executable files, manual, reference paper and support programs for the Survival Kit can be found at: http://www.nas.boku.ac.at/nuwi/the-survival-kit/ \cr
The preferred citation of the software including the usage of the R interface is: \cr
Meszaros, G., Soelkner, J., Ducrocq, V. (2013): The Survival Kit: Software to analyze survival data including possibly correlated random effects. Computer Methods and Programs in Biomedicine, 110(3): 503-510. DOI:10.1016/j.cmpb.2013.01.010
}
