# This is to demonstrate the Survival Kit R interface. 
# Before actual usage the package should be installed. Multiple possibilities for this, for example:
## Linux: R CMD INSTALL -l <directory with libraries> SurvKit4R_1.a.tar.gz
## Windows: click Packages > Install package(s) from local zip files... > find directory and install
# Lines such as this on beginning with a hash tag # are comments and therefore not used in R


#removes everything from R
rm(list=ls())

# UPDATE THE WORKING DIRECTORY HERE!!! IT MUST CONTAIN THE DATA FILES AND 
#  THE SURVIVAL KIT EXECUTABLE FILES (prepare.exe, weibull.exe, cox.exe with these file names)
# 
setwd("c:\\myFolder\\mySubfolder\\myData\\")

#load the SurvKit4R package
library("SurvKit4R")

#see within R help for:
# the main function: ?SKit4R
# Prepare function: ?prepare4R
# Cox and Weibull function: ?coxWei4R
# data sort function: ?sortSKit
# function for plotting Gram-Charlier approximation: ?GramChar

#note: you can skip time, ID and censoring specifications if you call:
#  the ID variable as "idnum"
#  the length of life variable "life"
#  the censoring variable "cens" and the censored values denoted as 0
# These are the default values in the R interface and can simplify the input as presented
#  in the example below. If you wish to use other names or coding you have to specify
#  additional function arguments. See help files within R. 


# all function arguments shold be followed by a comma, except the last one

# to run the R interface (Lung cancer example from the Survival Kit)
SKit4R(   
  verbose=TRUE,
### parameters for Prepare
  nrecmax=-1000,nefmax=50, ndimax=-50,ite_quasi=500,
  inputData="lung.dat", 
  inputSurvival="lung.fut",
  survivalOptions="QUANTILE 75 50 25 SPECIFIC 100 365",
  effects=c("idnum","life","treat","cell","cens","kps","age","diagtim","prior"),
  classEffects=c("treat","cell","prior"),
### parameters for Cox or Weibull
  program="weibull",
  title="Survival Kir R interface demonstration",
  model=c("treat","prior")
 #baseline=TRUE,
 #kaplan=TRUE
)

# You get the computation details in "logfile.txt" and 
#  the results in the "resultsMain.txt" file in your working directory

# Note: There is an experience of getting strange character fonts in text outputs in Linux.
#       If this happens (e.g. with gedit), try a different text editor to view. 
