      module parinclu2 
c***********************************************************************
c   Definition of parameters to used in the various programs of the    *
c                            SURVIVAL KIT                              *
c***********************************************************************
      IMPLICIT NONE
c
c
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c  PARAMETERS THAT MAY BE MODIFIED BY THE USER
c  Remember: when you change any of the parameters, you have to recompile
c  your programs.
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c
c
c Parameters related to the size of your problem
c ==============================================
c
      INTEGER INTE_ST
c
c-- NRECMAX = maximum number of elementary records
c-- (Caution: with time-dependent variables this number may be much  
c-- larger than the number of records in your original data file!!)
c
      integer :: NRECMAX=100000
c
c-- NRECMAX2 = maximum number of elementary records in the 'future'
c--    data file (FUTURE statement in PREPARE).
c--    These will normally be much fewer than in the data file
c
!      integer :: NRECMAX2=20000
! by default nrecmax2=1, if (NSURV.NE.0) nrecmax2=nrecmax
       integer nrecmax2
c
c-- MXEF = max. number of covariates. These are discrete
c--    (CLASS) covariates and continuous covariates.
c--    Time dependent covariates have to be counted twice (because
c--    the states before and after change occupy two positions on
c--    the recoded data file) !!
c
      integer :: MXEF=25
c
c-- MXEF_USED = max. number of covariates actually used in the
c--    MODEL statement of COX or WEIBULL.
c--    There may be situations where you create recoded files with
c--    very many covariates with PREPARE and never use all of them
c--    together in COX or WEIBULL. In this case it may make sense
c--    to have MXFE_USED smaller that MXEF (it must not be larger!)
c--    to save core memory space.
c
!      from Survival Kit v6 onwards just defined here, 
!       the exact value is computed in the program
      integer MXEF_USED
c

c-- MXSTRA= max. number of strata, i.e. levels of the strata variable
c--    defined in the STRATA statement.
c
      integer :: MXSTRA=25
c
c-- NDIMAX = max. total number of levels of effects in the model
c--    (size of the vector of solutions)
c
c     PARAMETER(NDIMAX=18000)
      integer :: NDIMAX=2000
c
c-- NDIMAX2 = this parameter is used to indicate whether the Hessian
c--    matrix is to be stored or not. It is used in COX and (WEIBULL with the
c--    DENSE_HESSIAN statement).
c--    The Hessian is needed to calculate standard deviations of estimated
c--    effects (STD_ERROR statement) and to find dependencies (option FIND
c--    in the CONSTRAINT statement).
c--    In WEIBULL, the Hessian is normally stored in sparse form (see
c--    parameters AVAIL and NZE below) except when stating DENSE_HESSIAN
c--    ndimax2 = ndimax -> Hessian is needed (and fully stored)
c--    ndimax2 = 1      -> Hessian is not needed (or sparsely stored)
c
c--   vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
c--   |  CAUTION:  this is likely to be a limiting parameter for the size  |
c--   |  of your problem. An incorrect specification of this parameter is  |
c--   |  is also one of the most frequent sources of errors when using the |
c--   |  Survival Kit !!                                                   |
c--   vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
c
c      PARAMETER(NDIMAX2=1)
!        PARAMETER(NDIMAX2=NDIMAX)
! specification moved to beginning of main
!   >>> BY DEFAULT: NDIMAX2=NDIMAX -- CHANGABLE BY KEYWORD: NO_HESS
      integer:: NDIMAX2  
c
c-- NTIMMAX = largest possible value of the (dependent) time variable
c--    this value is necessary as an upper limit for an efficient
c--    comutation of log(time) and exp(time) and when calculating
c--    functional values of the survivor curve (SURVIVAL statement
c--    of COX and WEIBULL)
c
      integer :: NTIMMAX=6000
c
c-- NSTIMAX = maximum number of distinct times or quantiles that
c--    can be defined in the SURVIVAL statement options SPECIFIC
c--    and QUANTILE.
c
      integer :: NSTIMAX=100
c
c-- INTE_ST gives the possibility to integrate out one loggamma
c--     random effect with the option INTEGRATE_OUT ... WITHIN ...
c--     WHILE a STRATA statement is specified 
c--     INTE_ST=4 ==> usual case 
c--     INTE_ST=5 ==>  STRATA + INTEGRATE_OUT .. WITHIN ... 
c
      PARAMETER(INTE_ST=4)
c
!      INTEGER BLKSIZE,BLKSIZE0,BLKSIZE2
!      PARAMETER(BLKSIZE0=(MXEF+3)*NRECMAX)
!      PARAMETER(BLKSIZE=(MXEF_USED+INTE_ST)*NRECMAX)
!      PARAMETER(BLKSIZE2=(MXEF+4)*NRECMAX2)
c
c Parameters used in the lbfgs maximisationn routine
c ==================================================
c
      INTEGER ITER_BF
      REAL*8 EPS_BFDEF

c-- MVEC_BF = number of vectors used to store the approximation of the Hessian
c--    (rank of approx. Hessian = MVEC_BF). Values between 3 and 20 might be
c--    used).
c
      integer :: MVEC_BF= 15
c
c-- EPS_BSDEF = Default convergence criterion, may be overridden using
c--    the CONVERGENCE_CRITERION statement in COX or WEIBULL.
c--    (||GRADIENT||/FUNCTION < EPS_BS max(1,||BETA||)
c
      PARAMETER(EPS_BFDEF=1.D-8)
c
c-- ITER_BF = maximum number of iterations.
c
      PARAMETER(ITER_BF= 1000)
c
c
c Parameters used in the numerical integration to get moments of the
c marginal posterior of a random effect variance parameter
c (when the MOMENTS option in the RANDOM statement is used)
c===================================================================
c
c-- NPGAUSS = number of points used during Gauss-Hermite integration
c      (has to be between 3 and 5)
c
      integer :: NPGAUSS= 5
c
c-- NITER_GAUSS = number of iterations in the Gauss-Hermite integration
c     process
C
      integer :: NITER_GAUSS= 5
c
c Parameters related to the parameter and report files in COX and WEIBULL
c =======================================================================
c
      LOGICAL UPPER_CASE,DETAIL_IT
      LOGICAL SUPPLY_INPUT1,SUPPLY_INPUT2
      CHARACTER*100 TEMPDIR,BACKUP
c
c-- upper_case = .TRUE. if all words (including file names) used in the
      PARAMETER(UPPER_CASE=.false.)
c
c-- SUPPLY_FILE1 : logical variable = .true. if the name of parameter file 1
c--    (provided by PREPARE) is supplied at running time (.false. if supplied
c--    in parinclu)
c
      PARAMETER (SUPPLY_INPUT1=.false.)
c
c-- parm1 : name of parameter file 1 (only if supply_file1=.false.)
c
      CHARACTER*100 :: parm1='varlist.txt'
c
c-- SUPPLY_FILE2 : logical variable = .true. if the name of parameter file 2
c--    is supplied at running time (.false. if supplied in parinclu)
c
      PARAMETER (SUPPLY_INPUT2=.false.)
c
c-- parm2 : name of parameter file 2 (used only if supply_file2=.false.)
c
      CHARACTER*100 :: parm2='cox.txt'
c
c-- SUPPLY_OUTPUT : logical variable = .true. if the name of the detailed output
c--    (with information about iterations) is supplied at running time
c
      logical :: SUPPLY_OUTPUT=.true.
c
c-- parmout : name of output file (used only if supply_output=.false.)
c
      CHARACTER*100 :: parmout='logfile.txt'
c
c-- TEMPDIR : name of the directory used for temporary files
c--        (= ' '  if = current directory)
c
      PARAMETER (TEMPDIR=' ')
c
c-- BACKUP  : name of the file used to backup solutions. Solutions
c--    will be stored after each iteration and may be used to restart
c--    the iteration procedure in case the machine breaks down during
c--    the maximisation (you have to use the READ_OLD_SOLUTIONS statement
c--    of COX or WEIBULL then). Only useful for very large applications.
c--    backup = ' ' if no backup file should be produced.
c
      PARAMETER (BACKUP=' ')
c
c-- DETAIL_IT  : switch on (if = .true.) or off (if =.false.) the  
c--    printing of detailed information (for each effect) at each iteration    
c--    of the maximization subroutine (average,mean or max absolute change
c--    between iterations)
      PARAMETER (DETAIL_IT=.false.)
c
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c  PARAMETERS THAT MAY _NOT_ BE MODIFIED BY THE USER
c  Don't you dare look beyond this point !!! :-)
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c
c Parameters related to the reading and translation of parameter files
c ====================================================================
c
      INTEGER MAXWORD,MAXLENG,MAXFIG,LRECL,NBUG_2000,NKEYC
c
c-- maxword = max. number of words in the parameter file
c-- maxleng = max. number of characters in a word
c-- maxfig = max. number of figures (i.e. numbers) in the parameter file
c-- lrecl = number of columns read per record
c-- nbug_2000 = maximum year value (yy=00==>nbug_2000) under which a D6 date
c--            is transformed from ddmmyy to ddmm(100+yy) to avoid the 'year 2000 bug'
c-- parameter files should be automatically transformed to upper-case names  
c-- nkeyc = number of keywords used in the parameter file for COX
      PARAMETER(MAXWORD=1000)
      PARAMETER(MAXLENG=40)
      PARAMETER(MAXFIG=400)
      PARAMETER(LRECL=150)
      PARAMETER(NBUG_2000=40)
      PARAMETER(NKEYC=75)
	 
! files transfered from common blocks
!! integers
      INTEGER ITOEV,NCOVMOD,ISTRVAR,IOMODE,NSTIME,NALL_TIM,NQUANT
      INTEGER NTOT,NRAND,MAXRAND,NPEST
      INTEGER IR1,IR2,NSTRATA,NSTOR
      INTEGER NRULE,NRELMAT,NANI,NRELMAT2
      INTEGER KFLAG,NOREP,KERFL,KW
      INTEGER NTRACE,MATRX,MASK(20)
      INTEGER NFMAX,NFLAT,JVARY,NXTRA,NPARMI
      INTEGER NSTR,NDIM,INDXS2(20)
! wn (word number) and kn(keyword number) to replace corresponding I-s and J-s
! The same as in prepare and weibull
      integer wn,kn
! flag varible for LOGFILE
      integer outflag
	  
!! reals
      REAL TOTIM
      REAL*8 EPS_BF,FOBJ,XXS(20)
      REAL*8 XMAX(20),XMIN(20),DELTX(20),DELMIN(20)
	  
!! characters
      CHARACTER*100 FILE(10)
	 
      REAL*8,allocatable:: H(:,:)  ! H(NDIMAX2,NDIMAX2)
	  
      CHARACTER*8 KEYWORD(NKEYC)
      integer kx
C-----------------------------------------------------------------------
C LIST OF KEYWORDS
C-----------------------------------------------------------------------
      DATA (KEYWORD(kx),kx=1,NKEYC)/'TIME    ','CODE    ','COVARIAT',
     +'STORAGE ','ON_DISK ','IN_CORE ','STRATA  ','MODEL   ',
     +'BASELINE','KAPLAN  ','TEST    ','GENERAL ','EFFECT  ',
     +'SEQUENTI','LAST    ','STD_ERRO','STORE_SO','READ_OLD',
     +'SURVIVAL','RESIDUAL','ALL_TIME','EQUAL_SP','SPECIFIC',
     +'QUANTILE','CONSTRAI','IMPOSE  ','CHECK   ','FIND    ',
     +'LARGEST ','NONE    ','RANDOM  ','LOGGAMMA','NORMAL  ',
     +'MULTINOR','USUAL_RU','MGS_RULE','SIRE_DAM','ESTIMATE',
     +'MOMENTS ','CONVERGE','STORE_BI','READ_BIN','ID      ',
     +'RHO_FIXE','DENSE_HE','INTEGRAT','JOINT_MO','INCLUDE_',
     +'PREVIOUS','STATISTI','ONLY_STA','JOINCODE','COEFFICI',
     +'FREE_FOR','FIXED_FO','UNFORMAT','BLOCKED_','COMPRESS',
     +'STD     ','FILES   ','DISCRETE','NTIMMAX ','NSTIMAX ',
     +'NVEC_BF ','NPGAUSS ','NITER_GA','NRECMAX ','NEFMAX  ',
     +'NSTRAMAX','NDIMAX  ','NO_HESSI','LOGFILE ','USE_R   ',
     +'CORRELAT','PRECISIO'/
	  
	  
! allocatable variables
      integer,allocatable :: istime(:)    !ISTIME(NSTIMAX) 
      real*8,allocatable :: squant(:)     !SQUANT(NSTIMAX)
      real*8,allocatable :: xjump(:)      !XJUMP(NTIMMAX)	  
      integer,allocatable :: IRANK(:)      !IRANK(NDIMAX)
      integer,allocatable :: IOLDCODE(:,:) !IOLDCODE(NDIMAX,4)
      integer,allocatable :: IOLCODST(:,:) !IOLCODST(MXSTRA,3)
      integer,allocatable :: NFAIL(:)      !NFAIL(NDIMAX)
      integer,allocatable :: ICONS(:)      !ICONS(0:NDIMAX)
      integer,allocatable :: IPEDIG(:,:)   !IPEDIG(4,NDIMAX)
      real*8,allocatable :: BETA(:)	       !BETA(NDIMAX)	
      real*8,allocatable :: DC_RISK(:)	       !DC_RISK(NDIMAX)	
      real*8,allocatable :: HESS(:,:),D2C_RISK(:,:)  !HESS(NDIMAX2,NDIMAX2)	

      integer,allocatable::ICOVTYP(:),NCOL(:),IMOD(:),ITRAIT1(:),IND(:) !all MXEF
      integer,allocatable :: ITRAIT2(:),IFIRST(:),ILAST(:),ITEST(:)       !all MXEF
      integer,allocatable :: IANAL1(:),IANAL2(:),IPLUS1(:),IPLUS2(:)      !all MXEF
      integer,allocatable :: NCOEF(:),ICOEF(:),ISAME(:)                   !all XX(0:MXEF)
      integer,allocatable :: IGAM(:,:),INOR(:,:)                          !IGAM(0:MXEF,3),INOR(0:MXEF,3)
      real*8,allocatable :: XCOEF(:),XAV(:),XSQ(:)                        !all MXEF
      real*8,allocatable :: XGAM(:),XNOR(:)                !XGAM(0:MXEF),XNOR(0:MXEF)
      real*8,allocatable :: BOUND(:,:)                     !BOUND(MXEF,3)
      character*8,allocatable :: LABEL(:)                  !LABEL(0:MXEF)

! to replace EQUIVALENCE statements - variables from common block DATA
      type blockdef
         integer, dimension (:,:),pointer :: info          !INFO(3,NRECMAX)
         integer, dimension (:,:),pointer :: idata         !IDATA(MXEF_USED,NRECMAX)
         real, dimension(:,:),pointer :: xdata             !XDATA(MXEF_USED,NRECMAX)
      end type blockdef
      type(blockdef) ::blockb
      integer NREC
	  
! to replace EQUIVALENCE statements - variables from common block DATA2
      type blockdef2
         integer, dimension (:,:),pointer :: info2          !INFO2(4,NRECMAX2)
         integer, dimension (:,:),pointer :: idata2         !IDATA2(MXEF_USED,NRECMAX2)
         real, dimension(:,:),pointer :: xdata2             !XDATA2(MXEF_USED,NRECMAX2)
      end type blockdef2
      type(blockdef2) ::blockb2
      integer NREC2
	  
! variable to specify if the R interface should be used
! changeable by keyword USE_R
      logical :: useR=.false.
	  
! CORRELATED RANDOM EFFECTS	
!  
! the value of correlation coeficient - initially no correlation assumed
      real*8 :: corr=0.d0
      real*8 :: precorr_mul=1.d0
! flag if the CORRELATION keyword is present - initially no correlation assumed 
!                                              no estimation erquired
      logical :: corrflag=.false.,estcor=.false.
! variables related to the correlated random effects
      integer,allocatable ::  icorr(:,:)   !icorr(MXEF,2)
      real*8 zeta     ! transformed value for correlation (between +- infinity)
      real*8 VM1(2,2) ! inverse or the matrix holding variances and correlation (V-1)
      real*8 detv     ! determinant of the V matrix	  

      end module parinclu2
