      SUBROUTINE CHOLESKY(A, LDA, N, INFO,ICODE)
C   (INITIALLY CALLED DPOFA)
C***PURPOSE  Factor a real symmetric positive definite matrix.
C      Modified (vpd 16/12/95) to accept semi-positive definite matrix
C***LIBRARY   SLATEC (LINPACK) (with CHANGES)
C
C***
C     CHOLESKY factors a double precision symmetric positive definite
C     matrix.
C
C     On Entry
C
C        A       DOUBLE PRECISION(LDA, N)
C                the symmetric matrix to be factored.  Only the
C                diagonal and upper triangle are used.
C
C        LDA     INTEGER
C                the leading dimension of the array  A .
C
C        N       INTEGER
C                the order of the matrix  A .
C
C     On Return
C
C        A       an upper triangular matrix  R  so that  A = TRANS(R)*R
C                where  TRANS(R)  is the transpose.
C                The strict lower triangle is unaltered.
C                If  INFO .NE. 0 , the factorization is not complete.
C
C        INFO    INTEGER
C                = 0  for normal return.
C                = K  signals an error condition.  The leading minor
C                     of order  K  is not positive definite.
C
      use parinclu2
      implicit none
c
      REAL*8 TOL
      PARAMETER(TOL=1.D-6)
C
      INTEGER LDA,N,INFO,I,J,JM1,K,JJ,ICODE
      REAL*8  A(LDA,*), DDOT,T,S
C***
         DO 30 J = 1, N
            INFO = J
            S = 0.0D0
            JM1 = J - 1
            IF (JM1 .LT. 1) GO TO 20
            DO 10 K = 1, JM1
               IF (A(K,K).EQ.0.D0) THEN
                    A(K,J)=0.D0
               ELSE
                   T = A(K,J) - DDOT(K-1,A(1,K),1,A(1,J),1)
                   T = T/A(K,K)
                   A(K,J) = T
                   S = S + T*T
                 ENDIF
   10       CONTINUE
   20       CONTINUE
            S = A(J,J) - S
            IF (S .LE. -TOL) THEN
                WRITE(6,100) J,S
  100         FORMAT( 'THE MATRIX IS NOT SEMI-POSITIVE DEFINITE',/1X,
     +        ' IN LIGN ', I5,' THE SQUARE ROOT OF ',D12.4,
     +        ' IS COMPUTED',/1X,
     +        ' THIS MAY BE DUE TO INCORRECTLY SPECIFIED CONSTRAINTS')
               RETURN
             ELSE IF (S.LE.TOL) THEN
                A(J,J) = 0.D0
                IF (ICODE.EQ.1) THEN
                   JJ=0
                   DO 25 I=1,ICONS(0)
                     IF (ICONS(I).EQ.J) JJ=1
   25              CONTINUE
                   IF (JJ.EQ.0) THEN
                     ICONS(0) = ICONS(0) + 1
                     ICONS(ICONS(0))= J
                     DO 28 I = J+1,N
                       A(J,K) = 0.D0
   28                CONTINUE
                  ENDIF
                ENDIF
             ELSE
                A(J,J) = SQRT(S)
            ENDIF
   30    CONTINUE
         INFO = 0
   40 CONTINUE
      RETURN
      END
       SUBROUTINE INVDET(A, LDA, N, DLDET, JOB)
C***(initially called DPODI)
C***PURPOSE  Compute the log-determinant and/or inverse of a certain real
C            symmetric positive definite matrix using the factors
C            computed by CHOLESKY.
C***LIBRARY   SLATEC (LINPACK) (with CHANGES)
C
C     INVDET computes the (non zero) logdeterminant and (generalized) inverse of a certain
C     double precision symmetric positive definite matrix (see below)
C     using the factor CHOLESKY.
C
C     On Entry
C
C        A       DOUBLE PRECISION(LDA, N)
C                the output  A  from CHOLESKY
C
C        LDA     INTEGER
C                the leading dimension of the array  A .
C
C        N       INTEGER
C                the order of the matrix  A .
C
C        JOB     INTEGER
C                = 11   both determinant and inverse.
C                = 01   inverse only.
C                = 10   determinant only.
C
C     On Return
C
C        A     INVDET produces the upper half of INVERSE(A) .
C                Elements of  A  below the diagonal are unchanged.
C                If the units digit of JOB is zero,  A  is unchanged.
C
C        DLDET     DOUBLE PRECISION
C                determinant of  A if requested.
C                Otherwise not referenced.)
C
      use parinclu2
      implicit none
      INTEGER LDA,N,JOB, I,J,JM1,K,KP1
      DOUBLE PRECISION A(LDA,*), DLDET,T
C
C     COMPUTE LOG DETERMINANT SKIPPING ZERO DIAGONALS
C
      IF (JOB.GE.10) THEN
         DLDET=0.0D0
         DO 50 I = 1, N
           IF (A(I,I).GT.0.D0) DLDET = DLDET+DLOG(A(I,I))
   50    CONTINUE
         DLDET = 2.D0 * DLDET
      ENDIF
        IF (MOD(JOB,10) .EQ. 0) RETURN
C
C     COMPUTE INVERSE(R)
C
         DO 100 K = 1, N
            IF (A(K,K).NE.0.D0) THEN
               A(K,K) = 1.0D0/A(K,K)
               T = -A(K,K)
               CALL DSCAL(K-1,T,A(1,K),1)
               KP1 = K + 1
               IF (N .LT. KP1) GO TO 70
               DO 60 J = KP1, N
                 T = A(K,J)
                 A(K,J) = 0.0D0
                 CALL DAXPY(K,T,A(1,K),1,A(1,J),1)
   60         CONTINUE
   70       CONTINUE
              ELSE
                KP1 = K + 1
                IF (N .LT. KP1) GO TO 90
                DO 80 J = KP1, N
                  A(K,J) = 0.0D0
   80         CONTINUE
             ENDIF
   90       CONTINUE

  100    CONTINUE
C
C        FORM  INVERSE(R) * TRANS(INVERSE(R))
C
         DO 130 J = 1, N
            JM1 = J - 1
            IF (JM1 .LT. 1) GO TO 120
            DO 110 K = 1, JM1
               T = A(K,J)
               CALL DAXPY(K,T,A(1,J),1,A(1,K),1)
  110       CONTINUE
  120       CONTINUE
            T = A(J,J)
            CALL DSCAL(J,T,A(1,J),1)
  130    CONTINUE
  140 CONTINUE
      RETURN
      END
      SUBROUTINE DSCAL (N, DA, DX, INCX)
C***
C***PURPOSE  Multiply a vector by a constant.
C***LIBRARY   SLATEC (BLAS)
C
C                B L A S  Subprogram
C    Description of Parameters
C
C     --Input--
C        N  number of elements in input vector(s)
C       DA  double precision scale factor
C       DX  double precision vector with N elements
C     INCX  storage spacing between elements of DX
C
C     --Output--
C       DX  double precision result (unchanged if N.LE.0)
C
C     Replace double precision DX by double precision DA*DX.
C     For I = 0 to N-1, replace DX(IX+I*INCX) with  DA * DX(IX+I*INCX),
C     where IX = 1 if INCX .GE. 0, else IX = 1+(1-N)*INCX.
C
      implicit none
       DOUBLE PRECISION DA, DX(*)
      INTEGER I, INCX, IX, M, MP1, N
C
      IF (N .LE. 0) RETURN
      IF (INCX .EQ. 1) GOTO 20
C
C     Code for increment not equal to 1.
C
      IX = 1
      IF (INCX .LT. 0) IX = (-N+1)*INCX + 1
      DO 10 I = 1,N
        DX(IX) = DA*DX(IX)
        IX = IX + INCX
   10 CONTINUE
      RETURN
C
C     Code for increment equal to 1.
C
C     Clean-up loop so remaining vector length is a multiple of 5.
C
   20 M = MOD(N,5)
      IF (M .EQ. 0) GOTO 40
      DO 30 I = 1,M
        DX(I) = DA*DX(I)
   30 CONTINUE
      IF (N .LT. 5) RETURN
   40 MP1 = M + 1
      DO 50 I = MP1,N,5
        DX(I) = DA*DX(I)
        DX(I+1) = DA*DX(I+1)
        DX(I+2) = DA*DX(I+2)
        DX(I+3) = DA*DX(I+3)
        DX(I+4) = DA*DX(I+4)
   50 CONTINUE
      RETURN
      END
      FUNCTION GAMLOG(G)
C***********************************************************************
C  GAMLOG        *     VINCENT DUCROCQ                *  28 / 02 /1990 *
C***********************************************************************
C  $$$$$$$$  CALCULATION OF LOG GAMMA(G)                 $$$$$$$$      *
C        ALGORITHMS 6.1.15, 6.1.36 AND 6.1.41 OF ABRAMOWITZ + STEGUN   *
C                                                                      *
C***********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      REAL*8 G,GAMLOG,G2,G3,A,B(8),H
C
      DATA TPI/0.9189385332D0/
      DATA (B(K),K=1,8)/-0.577191652D0,0.988205891D0,
     +-0.897056937D0,0.918206857D0,-0.756704078D0,0.482199394D0,
     +-0.193527818D0,0.035868343D0/
C
      IF (G.LE.1.D0) THEN
       GAMLOG=0.D0
       RETURN
      ELSE IF (G.GT.3.D0) THEN
       GAMLOG=(G-0.5D0)*DLOG(G)-G+TPI+1.D0/(12.D0*G)
       G2=G*G
       G3=G2*G
       A=360D0*G3
       IF (A.LT.1.D20) THEN
            GAMLOG=GAMLOG-1.D0/A
            G3=G3*G2
            A=1260D0*G3
            IF (A.LT.1.D20) THEN
             GAMLOG=GAMLOG+1.D0/A
             G3=G3*G2
             A=1680D0*G3
             IF (A.LT.1.D20) GAMLOG=GAMLOG-1.D0/A
            ENDIF
       ENDIF
       RETURN
      ELSE IF (G.LT.2) THEN
       H=G-1.D0
       GAMLOG=1.D0
  5      A=1.D0
       G2=1.D0
       DO 10 I=1,8
       G2=G2*H
       IF (G2.GT.1.D-20) A=A+B(I)*G2
 10      CONTINUE
       GAMLOG=DLOG(GAMLOG*A)
       RETURN
      ELSE
       H=G-2.D0
       GAMLOG=G-1.D0
       A=1.D0
       G2=1.D0
       DO 15 I=1,8
       G2=G2*H
       IF (G2.GT.1.D-20) A=A+B(I)*G2
 15      CONTINUE
       GAMLOG=DLOG(GAMLOG*A)
       RETURN
      ENDIF
C***********************************************************************
      END
      SUBROUTINE CHSCDF(X,NU,CDF)
C***********************************************************************
C     PURPOSE--THIS SUBROUTINE COMPUTES THE CUMULATIVE DISTRIBUTION
C              FUNCTION VALUE FOR THE CHI-SQUARED DISTRIBUTION
C              WITH INTEGER DEGREES OF FREEDOM PARAMETER = NU.
C              THIS DISTRIBUTION IS DEFINED FOR ALL NON-NEGATIVE X.
C              THE PROBABILITY DENSITY FUNCTION IS GIVEN
C              IN THE REFERENCES BELOW.
C     INPUT  ARGUMENTS--X      = THE SINGLE PRECISION VALUE AT
C                                WHICH THE CUMULATIVE DISTRIBUTION
C                                FUNCTION IS TO BE EVALUATED.
C                                X SHOULD BE NON-NEGATIVE.
C                     --NU     = THE INTEGER NUMBER OF DEGREES
C                                OF FREEDOM.
C                                NU SHOULD BE POSITIVE.
C     OUTPUT ARGUMENTS--CDF    = THE SINGLE PRECISION CUMULATIVE
C                                DISTRIBUTION FUNCTION VALUE.
C     OUTPUT--THE SINGLE PRECISION CUMULATIVE DISTRIBUTION
C             FUNCTION VALUE CDF FOR THE CHI-SQUARED DISTRIBUTION
C             WITH DEGREES OF FREEDOM PARAMETER = NU.
C     PRINTING--NONE UNLESS AN INPUT ARGUMENT ERROR CONDITION EXISTS.
C     RESTRICTIONS--X SHOULD BE NON-NEGATIVE.
C                 --NU SHOULD BE A POSITIVE INTEGER VARIABLE.
C     OTHER DATAPAC   SUBROUTINES NEEDED--NORCDF.
C     FORTRAN LIBRARY SUBROUTINES NEEDED--DSQRT, DEXP.
C---------------------------------------------------------------------
C
      IMPLICIT DOUBLE PRECISION(A-H,P-Z)
      REAL*8 DX,PI,CHI,SUM,TERM,AI,DCDFN
      REAL*8 DNU
      REAL*8 DSQRT,DEXP
      REAL*8 DLOG
      REAL*8 DFACT,DPOWER
      REAL*8 DW
      REAL*8 D1,D2,D3
      REAL*8 TERM0,TERM1,TERM2,TERM3,TERM4
      REAL*8 B11
      REAL*8 B21
      REAL*8 B31,B32
      REAL*8 B41,B42,B43
      DATA NUCUT/1000/
      DATA PI/3.14159265358979D0/
      DATA DPOWER/0.33333333333333D0/
      DATA B11/0.33333333333333D0/
      DATA B21/-0.02777777777778D0/
      DATA B31/-0.00061728395061D0/
      DATA B32/-13.0D0/
      DATA B41/0.00018004115226D0/
      DATA B42/6.0D0/
      DATA B43/17.0D0/
C
      IPR=6
C
C     CHECK THE INPUT ARGUMENTS FOR ERRORS
C
      IF(NU.LE.0)GOTO50
      IF(X.LT.0.0)GOTO55
      GOTO90
   50 WRITE(IPR,15)
      WRITE(IPR,47)NU
      CDF=0.0
      RETURN
   55 WRITE(IPR,4)
      WRITE(IPR,46)X
      CDF=0.0
      RETURN
   90 CONTINUE
    4 FORMAT(1H , 96H***** NON-FATAL DIAGNOSTIC--THE FIRST  INPUT ARGUME
     1NT TO THE CHSCDF SUBROUTINE IS NEGATIVE *****)
   15 FORMAT(1H , 91H***** FATAL ERROR--THE SECOND INPUT ARGUMENT TO THE
     1 CHSCDF SUBROUTINE IS NON-POSITIVE *****)
   46 FORMAT(1H , 35H***** THE VALUE OF THE ARGUMENT IS ,E15.8,6H *****)
   47 FORMAT(1H , 35H***** THE VALUE OF THE ARGUMENT IS ,I8   ,6H *****)
C
C-----START POINT-----------------------------------------------------
C
      DX=X
      ANU=NU
      DNU=NU
C
C     IF X IS NON-POSITIVE, SET CDF = 0.0 AND RETURN.
C     IF NU IS SMALLER THAN 10 AND X IS MORE THAN 200
C     STANDARD DEVIATIONS BELOW THE MEAN,
C     SET CDF = 0.0 AND RETURN.
C     IF NU IS 10 OR LARGER AND X IS MORE THAN 100
C     STANDARD DEVIATIONS BELOW THE MEAN,
C     SET CDF = 0.0 AND RETURN.
C     IF NU IS SMALLER THAN 10 AND X IS MORE THAN 200
C     STANDARD DEVIATIONS ABOVE THE MEAN,
C     SET CDF = 1.0 AND RETURN.
C     IF NU IS 10 OR LARGER AND X IS MORE THAN 100
C     STANDARD DEVIATIONS ABOVE THE MEAN,
C     SET CDF = 1.0 AND RETURN.
C
      IF(X.LE.1.D-20)GOTO105
      AMEAN=ANU
      SD=SQRT(2.0*ANU)
      Z=(X-AMEAN)/SD
      IF(NU.LT.10.AND.Z.LT.-200.0)GOTO105
      IF(NU.GE.10.AND.Z.LT.-100.0)GOTO105
      IF(NU.LT.10.AND.Z.GT.200.0)GOTO107
      IF(NU.GE.10.AND.Z.GT.100.0)GOTO107
      GOTO109
  105 CDF=0.0
      RETURN
  107 CDF=1.0
      RETURN
  109 CONTINUE
C
C     DISTINGUISH BETWEEN 3 SEPARATE REGIONS
C     OF THE (X,NU) SPACE.
C     BRANCH TO THE PROPER COMPUTATIONAL METHOD
C     DEPENDING ON THE REGION.
C     NUCUT HAS THE VALUE 1000.
C
      IF(NU.LT.NUCUT)GOTO1000
      IF(NU.GE.NUCUT.AND.X.LE.ANU)GOTO2000
      IF(NU.GE.NUCUT.AND.X.GT.ANU)GOTO3000
      IBRAN=1
      WRITE(IPR,99)IBRAN
   99 FORMAT(1H ,42H*****INTERNAL ERROR IN CHSCDF SUBROUTINE--,
     146HIMPOSSIBLE BRANCH CONDITION AT BRANCH POINT = ,I8)
      RETURN
C
C     TREAT THE SMALL AND MODERATE DEGREES OF FREEDOM CASE
C     (THAT IS, WHEN NU IS SMALLER THAN 1000).
C     METHOD UTILIZED--EXACT FINITE SUM
C     (SEE AMS 55, PAGE 941, FORMULAE 26.4.4 AND 26.4.5).
C
 1000 CONTINUE
      CHI=DSQRT(DX)
      IEVODD=NU-2*(NU/2)
      IF(IEVODD.EQ.0)GOTO120
C
      SUM=0.0D0
      TERM=1.0/CHI
      IMIN=1
      IMAX=NU-1
      GOTO130
C
  120 SUM=1.0D0
      TERM=1.0D0
      IMIN=2
      IMAX=NU-2
  130 IF(IMIN.GT.IMAX)GOTO160
      DO100I=IMIN,IMAX,2
      AI=I
      TERM=TERM*(DX/AI)
      SUM=SUM+TERM
  100 CONTINUE
  160 CONTINUE
      SUM=SUM*DEXP(-DX/2.0D0)
      IF(IEVODD.EQ.0)GOTO170
      SUM=(DSQRT(2.0D0/PI))*SUM
      SPCHI=CHI
      CALL NORCDF(SPCHI,CDFN)
      DCDFN=CDFN
      SUM=SUM+2.0D0*(1.0D0-DCDFN)
  170 CDF=1.0D0-SUM
      RETURN
C
C     TREAT THE CASE WHEN NU IS LARGE
C     (THAT IS, WHEN NU IS EQUAL TO OR GREATER THAN 1000)
C     AND X IS LESS THAN OR EQUAL TO NU.
C     METHOD UTILIZED--WILSON-HILFERTY APPROXIMATION
C     (SEE JOHNSON AND KOTZ, VOLUME 1, PAGE 176, FORMULA 28).
C
 2000 CONTINUE
      DFACT=4.5D0*DNU
      U=(((DX/DNU)**DPOWER)-1.0D0+(1.0D0/DFACT))*DSQRT(DFACT)
      CALL NORCDF(U,CDFN)
      CDF=CDFN
      RETURN
C
C     TREAT THE CASE WHEN NU IS LARGE
C     (THAT IS, WHEN NU IS EQUAL TO OR GREATER THAN 1000)
C     AND X IS LARGER THAN NU.
C     METHOD UTILIZED--HILL'S ASYMPTOTIC EXPANSION
C     (SEE JOHNSON AND KOTZ, VOLUME 1, PAGE 180, FORMULA 33.1).
C
 3000 CONTINUE
      DW=DSQRT(DX-DNU-DNU*DLOG(DX/DNU))
      DANU=DSQRT(2.0D0/DNU)
      D1=DW
      D2=DW**2
      D3=DW**3
      TERM0=DW
      TERM1=B11*DANU
      TERM2=B21*D1*(DANU**2)
      TERM3=B31*(D2+B32)*(DANU**3)
      TERM4=B41*(B42*D3+B43*D1)*(DANU**4)
      U=TERM0+TERM1+TERM2+TERM3+TERM4
      CALL NORCDF(U,CDFN)
      CDF=CDFN
      RETURN
C
      END
      SUBROUTINE NORCDF(X,CDF)
C
C     PURPOSE--THIS SUBROUTINE COMPUTES THE CUMULATIVE DISTRIBUTION
C              FUNCTION VALUE FOR THE NORMAL (GAUSSIAN)
C              DISTRIBUTION WITH MEAN = 0 AND STANDARD DEVIATION = 1.
C              THIS DISTRIBUTION IS DEFINED FOR ALL X AND HAS
C              THE PROBABILITY DENSITY FUNCTION
C              F(X) = (1/SQRT(2*PI))*EXP(-X*X/2).
C     INPUT  ARGUMENTS--X      = THE SINGLE PRECISION VALUE AT
C                                WHICH THE CUMULATIVE DISTRIBUTION
C                                FUNCTION IS TO BE EVALUATED.
C     OUTPUT ARGUMENTS--CDF    = THE SINGLE PRECISION CUMULATIVE
C                                DISTRIBUTION FUNCTION VALUE.
C     OUTPUT--THE SINGLE PRECISION CUMULATIVE DISTRIBUTION
C             FUNCTION VALUE CDF.
C
C---------------------------------------------------------------------
C
      IMPLICIT DOUBLE PRECISION(A-H,P-Z)
      DATA B1,B2,B3,B4,B5,P/.319381530,-0.356563782,1.781477937,-1.82125
     15978,1.330274429,.2316419/
C
      IPR=6
C
C     CHECK THE INPUT ARGUMENTS FOR ERRORS.
C     NO INPUT ARGUMENT ERRORS POSSIBLE
C     FOR THIS DISTRIBUTION.
C
C-----START POINT-----------------------------------------------------
C
      Z=X
      IF(X.LT.0.0)Z=-Z
      T=1.0/(1.0+P*Z)
      CDF=1.0-((0.39894228040143  )*EXP(-0.5*Z*Z))*(B1*T+B2*T**2+B3*T**3
     1+B4*T**4+B5*T**5)
      IF(X.LT.0.0)CDF=1.0-CDF
C
      RETURN
      END
      SUBROUTINE JSORT(IX,N,IY,IW,ORD)
C
C     PURPOSE--THIS SUBROUTINE SORTS
C              THE N ELEMENTS OF THE VECTOR IX
C              AND PUTS THE RESULTING N SORTED VALUES INTO THE
C              SINGLE PRECISION VECTOR IY.
C     INPUT  ARGUMENTS--IX      = THE  VECTOR OF
C                                OBSERVATIONS TO BE SORTED.
C                     --N      = THE INTEGER NUMBER OF OBSERVATIONS
C                                IN THE VECTOR IX.
C                     --IW     = WORKING VECTOR OF SIZE AT LEAST 2*N
C                     --ORD    = CHARACTER 'A' FOR ASCENDING
C                                          'D' FOR DESCENDING
C     OUTPUT ARGUMENTS--IY      = THE VECTOR
C                                INTO WHICH THE SORTED DATA VALUES
C                                FROM IX WILL BE PLACED.
C---------------------------------------------------------------------
C
      use parinclu2
      INTEGER I,J,K,L,M,N,JMI,JMK,MID,NM1,IP1,IPR,LMI
      INTEGER IW(*),IX(*),IY(*),IT,IMED,IHOLD
      CHARACTER ORD
C
      IPR=6
C
C     CHECK THE INPUT ARGUMENTS FOR ERRORS
C
      IF (N.LT.1)  GOTO  50
      IF (N.EQ.1)  GOTO  55
      IHOLD=IX(1)
      DO 60 I=2,N
        IF (IX(I).NE.IHOLD)  GOTO  90
   60 CONTINUE
      WRITE(IPR, 9) IHOLD
      DO 61 I=1,N
        IY(I)=IX(I)
   61 CONTINUE
      GOTO 500
   50 WRITE(IPR,15)
      WRITE(IPR,47) N
      GOTO 500
c   55 WRITE(IPR,18)
   55 IY(1)=IX(1)
      GOTO 500
   90 CONTINUE
    9 FORMAT(1H ,108H***** NON-FATAL DIAGNOSTIC--THE FIRST  INPUT ARGUME
     1NT (A VECTOR) TO THE SORT   SUBROUTINE HAS ALL ELEMENTS = ,E15.8,6
     1H *****)
   15 FORMAT(1H , 91H***** FATAL ERROR--THE SECOND INPUT ARGUMENT TO THE
     1 SORT   SUBROUTINE IS NON-POSITIVE *****)
   18 FORMAT(1H ,100H***** NON-FATAL DIAGNOSTIC--THE SECOND INPUT ARGUME
     1NT TO THE SORT   SUBROUTINE HAS THE VALUE 1 *****)
   47 FORMAT(1H , 35H***** THE VALUE OF THE ARGUMENT IS ,I8   ,6H *****)
C
C-----START POINT-----------------------------------------------------
C
C     COPY THE VECTOR X INTO THE VECTOR Y
      DO 100 I=1,N
        IY(I)=IX(I)
  100 CONTINUE
C
C     CHECK TO SEE IF THE INPUT VECTOR IS ALREADY SORTED
C
      NM1=N-1
      DO 200 I=1,NM1
        IP1=I+1
        IF (IY(I).LE.IY(IP1))  GOTO  200
        GOTO  250
  200 CONTINUE
      GOTO 500
  250 M=1
      I=1
      J=N
  305 IF (I.GE.J)  GOTO  370
  310 K=I
      MID=(I+J)/2
      IMED=IY(MID)
      IF (IY(I).LE.IMED)  GOTO  320
      IY(MID)=IY(I)
      IY(I)=IMED
      IMED=IY(MID)
  320 L=J
      IF (IY(J).GE.IMED)  GOTO  340
      IY(MID)=IY(J)
      IY(J)=IMED
      IMED=IY(MID)
      IF (IY(I).LE.IMED)  GOTO  340
      IY(MID)=IY(I)
      IY(I)=IMED
      IMED=IY(MID)
      GOTO  340
  330 IY(L)=IY(K)
      IY(K)=IT
  340 L=L-1
      IF (IY(L).GT.IMED)  GOTO  340
      IT=IY(L)
  350 K=K+1
      IF (IY(K).LT.IMED)  GOTO  350
      IF (K.LE.L)  GOTO  330
      LMI=L-I
      JMK=J-K
      IF (LMI.LE.JMK)  GOTO  360
      IW(M)=I
      IW(M+N)=L
      I=K
      M=M+1
       GOTO 380
  360 IW(M)=K
      IW(M+N)=J
      J=L
      M=M+1
       GOTO 380
  370 M=M-1
      IF (M.EQ.0) GOTO 500
      I=IW(M)
      J=IW(M+N)
  380 JMI=J-I
      IF (JMI.GE.11)  GOTO 310
      IF (I.EQ.1) GOTO 305
       I=I-1
  390  I=I+1
      IF (I.EQ.J) GOTO 370
      IMED=IY(I+1)
      IF (IY(I).LE.IMED)  GOTO 390
      K=I
  395 IY(K+1)=IY(K)
      K=K-1
      IF (IMED.LT.IY(K)) GOTO 395
      IY(K+1)=IMED
      GOTO  390
C  REORDER IF DESCENDING ORDER
  500 IF (ORD.EQ.'D') THEN
        DO 510 I=1,N/2
	      IT= IY(I)
	      J=N-I+1
	      IY(I) = IY(J)
	      IY(J) = IT
  510   CONTINUE
      ENDIF
      RETURN
      END
      SUBROUTINE DPSORT (DX, N, IPERM, KFLAG, IER)
C***BEGIN PROLOGUE  DPSORT
C***PURPOSE  Return the permutation vector generated by sorting a given
C            array and, optionally, rearrange the elements of the array.
C            The array may be sorted in increasing or decreasing order.
C            A slightly modified quicksort algorithm is used.
C***LIBRARY   SLATEC
C***CATEGORY  N6A1B, N6A2B
C***TYPE      DOUBLE PRECISION (SPSORT-S, DPSORT-D, IPSORT-I, HPSORT-H)
C***KEYWORDS  NUMBER SORTING, PASSIVE SORTING, SINGLETON QUICKSORT, SORT
C***AUTHOR  Jones, R. E., (SNLA)
C           Rhoads, G. S., (NBS)
C           Wisniewski, J. A., (SNLA)
C***DESCRIPTION
C
C   DPSORT returns the permutation vector IPERM generated by sorting
C   the array DX and, optionally, rearranges the values in DX.  DX may
C   be sorted in increasing or decreasing order.  A slightly modified
C   quicksort algorithm is used.
C
C   IPERM is such that DX(IPERM(I)) is the Ith value in the
C   rearrangement of DX.  IPERM may be applied to another array by
C   calling IPPERM, SPPERM, DPPERM or HPPERM.
C
C   The main difference between DPSORT and its active sorting equivalent
C   DSORT is that the data are referenced indirectly rather than
C   directly.  Therefore, DPSORT should require approximately twice as
C   long to execute as DSORT.  However, DPSORT is more general.
C
C   Description of Parameters
C      DX - input/output -- double precision array of values to be
C           sorted.  If ABS(KFLAG) = 2, then the values in DX will be
C           rearranged on output; otherwise, they are unchanged.
C      N  - input -- number of values in array DX to be sorted.
C      IPERM - output -- permutation array such that IPERM(I) is the
C              index of the value in the original order of the
C              DX array that is in the Ith location in the sorted
C              order.
C      KFLAG - input -- control parameter:
C            =  2  means return the permutation vector resulting from
C                  sorting DX in increasing order and sort DX also.
C            =  1  means return the permutation vector resulting from
C                  sorting DX in increasing order and do not sort DX.
C            = -1  means return the permutation vector resulting from
C                  sorting DX in decreasing order and do not sort DX.
C            = -2  means return the permutation vector resulting from
C                  sorting DX in decreasing order and sort DX also.
C      IER - output -- error indicator:
C          =  0  if no error,
C          =  1  if N is zero or negative,
C          =  2  if KFLAG is not 2, 1, -1, or -2.
C***REFERENCES  R. C. Singleton, Algorithm 347, An efficient algorithm
C                 for sorting with minimal storage, Communications of
C                 the ACM, 12, 3 (1969), pp. 185-187.
C           IF-THEN-ELSE-ENDIF.  (SMR, WRB)
C***END PROLOGUE  DPSORT
      implicit none
C     .. Scalar Arguments ..
      INTEGER IER, KFLAG, N
C     .. Array Arguments ..
      DOUBLE PRECISION DX(*)
      INTEGER IPERM(*)
C     .. Local Scalars ..
      DOUBLE PRECISION R, TEMP
      INTEGER I, IJ, INDX, INDX0, ISTRT, J, K, KK, L, LM, LMT, M, NN
C     .. Local Arrays ..
      INTEGER IL(21), IU(21)
C     .. External Subroutines ..
c      EXTERNAL XERMSG
C     .. Intrinsic Functions ..
      INTRINSIC ABS, INT
C***FIRST EXECUTABLE STATEMENT  DPSORT
      IER = 0
      NN = N
      IF (NN .LT. 1) THEN
         IER = 1
         write(6,*) 'DPSORT',
     +    'The number of values to be sorted, N, is not positive.'
         RETURN
      ENDIF
C
      KK = ABS(KFLAG)
      IF (KK.NE.1 .AND. KK.NE.2) THEN
         IER = 2
         write(6,*) 'DPSORT',
     +    'The sort control parameter, KFLAG, is not 2, 1, -1, or -2.'
         RETURN
      ENDIF
C
C     Initialize permutation vector
C
      DO 10 I=1,NN
         IPERM(I) = I
   10 CONTINUE
C
C     Return if only one value is to be sorted
C
      IF (NN .EQ. 1) RETURN
C
C     Alter array DX to get decreasing order if needed
C
      IF (KFLAG .LE. -1) THEN
         DO 20 I=1,NN
            DX(I) = -DX(I)
   20    CONTINUE
      ENDIF
C
C     Sort DX only
C
      M = 1
      I = 1
      J = NN
      R = .375D0
C
   30 IF (I .EQ. J) GO TO 80
      IF (R .LE. 0.5898437D0) THEN
         R = R+3.90625D-2
      ELSE
         R = R-0.21875D0
      ENDIF
C
   40 K = I
C
C     Select a central element of the array and save it in location L
C
      IJ = I + INT((J-I)*R)
      LM = IPERM(IJ)
C
C     If first element of array is greater than LM, interchange with LM
C
      IF (DX(IPERM(I)) .GT. DX(LM)) THEN
         IPERM(IJ) = IPERM(I)
         IPERM(I) = LM
         LM = IPERM(IJ)
      ENDIF
      L = J
C
C     If last element of array is less than LM, interchange with LM
C
      IF (DX(IPERM(J)) .LT. DX(LM)) THEN
         IPERM(IJ) = IPERM(J)
         IPERM(J) = LM
         LM = IPERM(IJ)
C
C        If first element of array is greater than LM, interchange
C        with LM
C
         IF (DX(IPERM(I)) .GT. DX(LM)) THEN
            IPERM(IJ) = IPERM(I)
            IPERM(I) = LM
            LM = IPERM(IJ)
         ENDIF
      ENDIF
      GO TO 60
   50 LMT = IPERM(L)
      IPERM(L) = IPERM(K)
      IPERM(K) = LMT
C
C     Find an element in the second half of the array which is smaller
C     than LM
C
   60 L = L-1
      IF (DX(IPERM(L)) .GT. DX(LM)) GO TO 60
C
C     Find an element in the first half of the array which is greater
C     than LM
C
   70 K = K+1
      IF (DX(IPERM(K)) .LT. DX(LM)) GO TO 70
C
C     Interchange these elements
C
      IF (K .LE. L) GO TO 50
C
C     Save upper and lower subscripts of the array yet to be sorted
C
      IF (L-I .GT. J-K) THEN
         IL(M) = I
         IU(M) = L
         I = K
         M = M+1
      ELSE
         IL(M) = K
         IU(M) = J
         J = L
         M = M+1
      ENDIF
      GO TO 90
C
C     Begin again on another portion of the unsorted array
C
   80 M = M-1
      IF (M .EQ. 0) GO TO 120
      I = IL(M)
      J = IU(M)
C
   90 IF (J-I .GE. 1) GO TO 40
      IF (I .EQ. 1) GO TO 30
      I = I-1
C
  100 I = I+1
      IF (I .EQ. J) GO TO 80
      LM = IPERM(I+1)
      IF (DX(IPERM(I)) .LE. DX(LM)) GO TO 100
      K = I
C
  110 IPERM(K+1) = IPERM(K)
      K = K-1
      IF (DX(LM) .LT. DX(IPERM(K))) GO TO 110
      IPERM(K+1) = LM
      GO TO 100
C
C     Clean up
C
  120 IF (KFLAG .LE. -1) THEN
         DO 130 I=1,NN
            DX(I) = -DX(I)
  130    CONTINUE
      ENDIF
C
C     Rearrange the values of DX if desired
C
      IF (KK .EQ. 2) THEN
C
C        Use the IPERM vector as a flag.
C        If IPERM(I) < 0, then the I-th value is in correct location
C
         DO 150 ISTRT=1,NN
            IF (IPERM(ISTRT) .GE. 0) THEN
               INDX = ISTRT
               INDX0 = INDX
               TEMP = DX(ISTRT)
  140          IF (IPERM(INDX) .GT. 0) THEN
                  DX(INDX) = DX(IPERM(INDX))
                  INDX0 = INDX
                  IPERM(INDX) = -IPERM(INDX)
                  INDX = ABS(IPERM(INDX))
                  GO TO 140
               ENDIF
               DX(INDX0) = TEMP
            ENDIF
  150    CONTINUE
C
C        Revert the signs of the IPERM values
C
         DO 160 I=1,NN
            IPERM(I) = -IPERM(I)
  160    CONTINUE
C
      ENDIF
C
      RETURN
      END
      SUBROUTINE XSORT(X,N,Y,IW,ORD)
C---------------------------------------------------------------------
C AS FOR JSORT BUT X AND Y ARE REAL*8 VECTORS
C---------------------------------------------------------------------
C
      use parinclu2
      implicit none
      INTEGER I,J,K,L,M,N,JMI,JMK,MID,NM1,IP1,IPR,LMI
      INTEGER IW(*)
      REAL*8 X(*),Y(*),T,AMED,HOLD
      CHARACTER ORD
C
      IPR=6
C
C     CHECK THE INPUT ARGUMENTS FOR ERRORS
C
      IF (N.LT.1)  GOTO  50
      IF (N.EQ.1)  GOTO  55
      HOLD=X(1)
      DO 60 I=2,N
        IF (X(I).NE.HOLD)  GOTO  90
   60 CONTINUE
      WRITE(IPR, 9) HOLD
      DO 61 I=1,N
        Y(I)=X(I)
   61 CONTINUE
      GOTO 500
   50 WRITE(IPR,15)
      WRITE(IPR,47) N
      GOTO 500
c   55 WRITE(IPR,18)
   55 Y(1)=X(1)
      GOTO 500
   90 CONTINUE
    9 FORMAT(1H ,108H***** NON-FATAL DIAGNOSTIC--THE FIRST  INPUT ARGUME
     1NT (A VECTOR) TO THE SORT   SUBROUTINE HAS ALL ELEMENTS = ,E15.8,6
     1H *****)
   15 FORMAT(1H , 91H***** FATAL ERROR--THE SECOND INPUT ARGUMENT TO THE
     1 SORT   SUBROUTINE IS NON-POSITIVE *****)
   18 FORMAT(1H ,100H***** NON-FATAL DIAGNOSTIC--THE SECOND INPUT ARGUME
     1NT TO THE SORT   SUBROUTINE HAS THE VALUE 1 *****)
   47 FORMAT(1H , 35H***** THE VALUE OF THE ARGUMENT IS ,I8   ,6H *****)
C
C-----START POINT-----------------------------------------------------
C
C     COPY THE VECTOR X INTO THE VECTOR Y
      DO 100 I=1,N
        Y(I)=X(I)
  100 CONTINUE
C
C     CHECK TO SEE IF THE INPUT VECTOR IS ALREADY SORTED
C
      NM1=N-1
      DO 200 I=1,NM1
        IP1=I+1
        IF (Y(I).LE.Y(IP1))  GOTO  200
        GOTO  250
  200 CONTINUE
      GOTO 500
  250 M=1
      I=1
      J=N
  305 IF (I.GE.J)  GOTO  370
  310 K=I
      MID=(I+J)/2
      AMED=Y(MID)
      IF (Y(I).LE.AMED)  GOTO  320
      Y(MID)=Y(I)
      Y(I)=AMED
      AMED=Y(MID)
  320 L=J
      IF (Y(J).GE.AMED)  GOTO  340
      Y(MID)=Y(J)
      Y(J)=AMED
      AMED=Y(MID)
      IF (Y(I).LE.AMED)  GOTO  340
      Y(MID)=Y(I)
      Y(I)=AMED
      AMED=Y(MID)
      GOTO  340
  330 Y(L)=Y(K)
      Y(K)=T
  340 L=L-1
      IF (Y(L).GT.AMED)  GOTO  340
      T=Y(L)
  350 K=K+1
      IF (Y(K).LT.AMED)  GOTO  350
      IF (K.LE.L)  GOTO  330
      LMI=L-I
      JMK=J-K
      IF (LMI.LE.JMK)  GOTO  360
      IW(M)=I
      IW(M+N)=L
      I=K
      M=M+1
       GOTO 380
  360 IW(M)=K
      IW(M+N)=J
      J=L
      M=M+1
       GOTO 380
  370 M=M-1
      IF (M.EQ.0) GOTO 500
      I=IW(M)
      J=IW(M+N)
  380 JMI=J-I
      IF (JMI.GE.11)  GOTO 310
      IF (I.EQ.1) GOTO 305
       I=I-1
  390  I=I+1
      IF (I.EQ.J) GOTO 370
      AMED=Y(I+1)
      IF (Y(I).LE.AMED)  GOTO 390
      K=I
  395 Y(K+1)=Y(K)
      K=K-1
      IF (AMED.LT.Y(K)) GOTO 395
      Y(K+1)=AMED
      GOTO  390
C  REORDER IF DESCENDING ORDER
  500 IF (ORD.EQ.'D') THEN
        DO 510 I=1,N/2
	    T= Y(I)
	    J=N-I+1
	    Y(I) = Y(J)
	    Y(J) = T
  510   CONTINUE
	ENDIF
	RETURN
      END

      SUBROUTINE JRANK(IX,N,IR,IW,ORD)
C
C     PURPOSE--THIS SUBROUTINE RANKS THE N ELEMENTS OF VECTOR IX,
C              AND PUTS THE RESULTING N RANKS INTO THE INTEGER VECTOR IR.

C     INPUT  ARGUMENTS--IX      = THE INTEGER VECTOR OF
C                                OBSERVATIONS TO BE RANKED.
C                     --N      = THE INTEGER NUMBER OF OBSERVATIONS
C                                IN THE VECTOR X.
C                     --IW     = WORKING INTEGER VECTOR OF SIZE AT LEAST 3N
C                     --ORD    = CHARACTER 'A' FOR ASCENDING
C                                          'D' FOR DESCENDING
C     OUTPUT ARGUMENTS--IR     = THE INTEGER VECTOR INTO WHICH THE RANKS
C                                FROM X WILL BE PLACED.
C
C---------------------------------------------------------------------
C
      use parinclu2
      implicit none
      INTEGER I,J,JP1,K,N,IPR,JMIN,NM1,IR(*),IW(*)
      INTEGER IX(*),AN,AVRANK,IRPREV,IUPPER,IBRAN
      INTEGER IHOLD,IXPREV
      CHARACTER ORD
C
      AN=N
      IPR=6
      IUPPER=N
C
C     CHECK THE INPUT ARGUMENTS FOR ERRORS
C
      IF (N.LT.1.OR.N.GT.IUPPER) GOTO 50
      IF (N.EQ.1) GOTO 55
      IHOLD=IX(1)
      DO 60 I=2,N
        IF (IX(I).NE.IHOLD) GOTO 90
   60 CONTINUE
      WRITE(IPR, 9) IHOLD
        AVRANK=(AN+1)/2
      DO 61 I=1,N
        IR(I)=AVRANK
   61 CONTINUE
      GOTO 500
   50 WRITE(IPR,17) IUPPER
      WRITE(IPR,47) N
      GOTO 500
C   55 WRITE(IPR,18)
   55 IR(1)=1
      GOTO 500
   90 CONTINUE
    9 FORMAT(1H ,109H***** NON-FATAL DIAGNOSTIC--THE FIRST  INPUT ARGUME
     1NT (A VECTOR) TO THE RANK   SUBROUTINE HAS ALL ELEMENTS = ,I8,6
     1H *****)
   17 FORMAT(1H , 98H***** FATAL ERROR--THE SECOND INPUT ARGUMENT TO THE
     1 RANK   SUBROUTINE IS OUTSIDE THE ALLOWABLE (1,,I6,16H) INTERVAL *
     1****)
   18 FORMAT(1H ,100H***** NON-FATAL DIAGNOSTIC--THE SECOND INPUT ARGUME
     1NT TO THE RANK   SUBROUTINE HAS THE VALUE 1 *****)
   47 FORMAT(1H , 35H***** THE VALUE OF THE ARGUMENT IS ,I8   ,6H *****)
C
C-----START POINT-----------------------------------------------------
C
C     FIRST SORT THE DATA FROM THE INPUT VECTOR X
C     INTO THE INTERMEDIATE STORAGE VECTOR XS.
C
      CALL JSORT(IX,N,IW(1),IW(AN+1),'A')
C
C     NOW DETERMINE THE RANKS. THE ALGORITHM IS TO TAKE A GIVEN ELEMENT
C     IN THE ORIGINAL VECTOR X, SCAN THE SORTED VALUES IN THE XS VECTOR
C     UNTIL A MATCH IS FOUND; THEN THE RANK FOR THAT
C     VALUE IN THE XS VECTOR IS DETERMINED.
C     THAT RANK IS THEN WRITTEN INTO THAT POSITION
C     IN THE OUTPUT Y VECTOR WHICH CORRESPONDS TO THE POSITION OF THE
C     GIVEN ELEMENT OF INTEREST IN THE ORIGINAL X VECTOR.
C     THE CODE IS LENGTHENED FROM THIS BASIC ALGORITHM
C     BY A SECTION WHICH CUTS DOWN THE SEARCH IN THE XS VECTOR,
C     AND BY A SECTION WHICH OBVIATES (UNDER CERTAIN CIRCUMSTANCES)
C     THE NEED FOR RECALCULATING THE RANK OF AN ELEMENT IN XS.
C
      NM1=N-1
      IXPREV=IX(1)
      DO 700 I=1,N
      JMIN=1
      IF (IX(I).GT.IXPREV) GOTO 770
      IF (I.EQ.1) GOTO 790
      IF (IX(I).EQ.IXPREV) GOTO 750
       GOTO 790
  750 CONTINUE
      IR(I)=IRPREV
       GOTO 880
  770 CONTINUE
      JMIN=K
      IF (JMIN.LT.N) GOTO 790
      IF (JMIN.EQ.N) GOTO 820
      IBRAN=1
      WRITE(IPR,109)IBRAN
      WRITE(IPR,101)JMIN
  109 FORMAT(1H ,40H*****INTERNAL ERROR IN RANK SUBROUTINE--,
     146HIMPOSSIBLE BRANCH CONDITION AT BRANCH POINT = ,I8)
  101 FORMAT(1H ,'JMIN = ',I8)
      STOP
  790 CONTINUE
      DO800J=JMIN,NM1
      IF (IX(I).NE.IW(J)) GOTO 800
      JP1=J+1
      DO 900 K=JP1,N
      IF (IW(K).NE.IW(J)) GOTO 950
  900 CONTINUE
      K=N+1
  950 CONTINUE
      AVRANK=J+K-1
      AVRANK=AVRANK/2
      IR(I)=AVRANK
       GOTO 880
  800 CONTINUE
  820 CONTINUE
      J=N
      K=N+1
      IF (IX(I).EQ.IW(J)) GOTO 850
      IBRAN=2
      WRITE(IPR,109) IBRAN
      WRITE(IPR,102) IX(I),IW(J)
  102 FORMAT(1H ,'IX(I) = ',I9,'   IW(J) = ',I9)
      STOP
  850 CONTINUE
      IR(I)=N
  880 CONTINUE
      IXPREV=IX(I)
      IRPREV=IR(I)
  700 CONTINUE
C  REORDER IF DESCENDING ORDER
  500 IF (ORD.EQ.'D') THEN
        DO 510 I=1,N/2
	      K= IR(I)
	      J=N-I+1
	      IR(I) = IR(J)
	      IR(J) = K
  510   CONTINUE
      ENDIF
      RETURN
      END
