      module prepinclu

c***********************************************************************
c   Definition of parameters to used in the various programs of the    *
c                            SURVIVAL KIT                              *
c***********************************************************************
      IMPLICIT NONE
c
      INTEGER LTAB0,LTAB1
      INTEGER*8 IDFILE,IDFILE2,LTAB02,LTAB12
      INTEGER INTFOR
      PARAMETER (INTFOR=4)
c
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c  PARAMETERS THAT MAY BE MODIFIED BY THE USER
c  Remember: when you change any of the parameters, you have to recompile
c  your programs.
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c
c
c Parameters related to the size of your problem
c ==============================================
c
c-- NRECMAX = maximum number of elementary records
c-- (Caution: with time-dependent variables this number may be much  
c-- larger than the number of records in your original data file!!)
c
      integer:: NRECMAX=100000
c
c-- MXEF = max. number of covariates. These are discrete
c--    (CLASS) covariates and continuous covariates.
c--    Time dependent covariates have to be counted twice (because
c--    the states before and after change occupy two positions on
c--    the recoded data file) !!
c
      integer:: MXEF=40
c
c-- NDIMAX = max. total number of levels of effects in the model
c--    (size of the vector of solutions)
c
      integer:: NDIMAX=10000
c        PARAMETER(NDIMAX=28000)
c
c-- NCOLPED is the number of columns (-1) for the pedigree file
c--    =3 if animal, sire, dam or mgs
c--    =4 if animal, sire, mgs, mgd
c
      integer:: NCOLPED=3

c Parameters relating to the binary storage of the initial file when 
c--       the statement STORE_BINARY is used  
c ======================================================================
c
c-- nrecbloc = number of records per block when writting/reading
c-- a binary file for fast I/O (statements STORE_BINARY and
c-- READ_BINARY)
c
c      integer:: NRECBLOC=100000
c      PARAMETER(NRECBLOC=NRECMAX)
c
c
c Parameters related to the parameter file of PREPARE
c ===================================================
c
         LOGICAL SUPPLY_INPUT0,UPPER_CASE
c
c-- upper_case = .TRUE. if all words (including file names) used in the
      PARAMETER(UPPER_CASE=.false.)
c
c-- Normally, the name of the parameter file will be requested by
c-- program PREPARE. The following parameters are designed to
c-- circumvent this request and start the program using a defined name
c-- for the parameter file
c
c-- SUPPLY_INPUT0 is a logical variable defining the way the name
c--    of the parameter file is supplied:
c--    supply_file=.true.  -> parameter file is supplied at running time
c--    supply_file=.false. -> name of parameter file is defined in parinclu
c
      PARAMETER (SUPPLY_INPUT0=.false.)
c
c-- parm1 = name of the parameter file (used only if supply_file0 = .false.)
c
      CHARACTER*100 :: parm1='prepare.txt'
c
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c  PARAMETERS THAT MAY _NOT_ BE MODIFIED BY THE USER
c  Don't you dare look beyond this point !!! :-)
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c
c Parameters related to the reading and translation of parameter files
c ====================================================================
c
      INTEGER MAXWORD,MAXLENG,MAXFIG,LRECL,NBUG_2000,NKEYFE
c
c-- maxword = max. number of words in the parameter file
c-- maxleng = max. number of characters in a word
c-- maxfig = max. number of figures (i.e. numbers) in the parameter file
c-- lrecl = number of columns read per record
c-- nbug_2000 = maximum year value (yy=00==>nbug_2000) under which a D6 date
c--            is transformed from ddmmyy to ddmm(100+yy) to avoid the 'year 2000 bug'
c-- parameter files should be automatically transformed to upper-case names  
c-- nkeyfe = number of keywords (i.e. statement types) used in the
c--    parameter file for PREPARE
      PARAMETER(MAXWORD=1000)
      PARAMETER(MAXLENG=40)
      PARAMETER(MAXFIG=400)
      PARAMETER(LRECL=150)
      PARAMETER(NBUG_2000=40)
      PARAMETER(NKEYFE=22)
  
!*************************************************************
! new part - type definitions and "globalization" of variables
!          previously in common blocks - Meszaros, 2008 - 2009
!*************************************************************
      integer idrev,timev(3),censv(2),trunv,futurf, idped(2)
      integer join(2),nrecr,nrecw,timcov(3),fout
      integer nrr,ninf,ltab,ntcmb,ntimdep,ir1,ir2
      integer infp(3),ntimdepr,NWORD,NVAR,nvread	
      integer new,nlev               !NEW and NLEV are also in funct. IDMURL, NLEV also in subroutine BITSTR2
      integer indl,ipla,lastil       !INDL, IPLA and LASTIL are also in funct. IDMURL
      !integer rtopl(0:mxef)         !RTOPL (now alloc.) is also in funct. IDMURL
      integer levcod(3)              !XLIST (now alloc.) and LEVCOD are also in funct. IDMURL, LEVCOD also in subroutine BITSTR2
      
      integer wn                     !introduced new variable "wn" as word number instead of "I" in WORD vector
      integer k                      ! introduced new variable "kn" as keyword number instead of "J" in KEYWORD vector
	  
      integer kk
	  
      character*5 fmi,fmr           !FMI and FMR are also in function CREATEFM in the main program
      character*8 keyword(nkeyfe)    
      CHARACTER*15 file(8)
      character*24 fm2,fm3

C LIST OF KEYWORDS
       DATA (KEYWORD(kk),kk=1,nkeyfe)/'FILES   ','NRECMAX ',
     +     'NEFMAX  ','NDIMAX  ','INPUT   ','TIME    ',
     +     'CENSCODE','IDREC   ','TRUNCATE','TIMECOV ','CONVDT  ',
     +     'CLASS   ','TIMEDEP ','COMBINE ','OUTPUT  ','FUTURE  ',
     +     'PEDIGREE','JOINCODE','FORMOUT ','DISCRETE','CORRELAT',
     +     'NCOLPED '/
	  
! Definition of fixed size arrays
      integer varcc(maxword),timc(maxfig),pir(maxword)
      integer IFIG(MAXFIG),LFIG(0:MAXWORD),NFIG(0:MAXWORD)
      integer vartim(maxword,2),varcom(maxword,4)
      character WORD(MAXLENG,0:MAXWORD)
      character*2 vartyp(maxword)
      character*8 varnam(maxword)
	  
! Definition of allocatable arrays
!! integers
      integer, allocatable, dimension(:) :: tcmb            !tcmb(mxef)
      integer, allocatable, dimension(:) :: rlev            !rlev(mxef) 
      integer, allocatable, dimension(:) :: tco             !tco(mxef)
      integer, allocatable, dimension(:) :: ir1p,ir2p       !ir1p(mxef),ir2p(mxef)
      integer, allocatable, dimension(:) :: rtopl           !rtopl(0:mxef)
      integer, allocatable, dimension(:) :: krr              !krr(mxef*nrecmax)	  
	  	  
      integer, allocatable, dimension(:,:) :: xlist         !xlist(ndimax,8)
      integer, allocatable, dimension(:,:) :: krec, iv      !krec(0:17,nrecmaxr) ; iv(nchlim+200,3)	  
      integer, allocatable, dimension(:,:) :: itabl         !itabl(nrecmax,mxef)

!! reals
      real, allocatable, dimension(:,:) :: xtabl             !xtabl(nrecmax,mxef)
	  
c variable to test order of keywords in parameter file
      logical:: parmark=.FALSE.
c variables needed when the CORRELATION keyword is used
      logical:: joinCor=.FALSE.
      logical:: corr=.FALSE.
      character*8:: corrName(2)
c variables to determine wether the keyword was used
      integer:: nrecmaxch,mxefch,ndimaxch,transfcount,ncolpedch
	  
      end module prepinclu
