      module parinclu
c***********************************************************************
c   Definition of parameters to used in the various programs of the    *
c                            SURVIVAL KIT                              *
c***********************************************************************
      IMPLICIT NONE
c
      INTEGER LTAB0,LTAB1
      INTEGER*8 IDFILE,IDFILE2,LTAB02,LTAB12
      INTEGER INTFOR
      PARAMETER (INTFOR=4)
c
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c  PARAMETERS THAT MAY BE MODIFIED BY THE USER
c  Remember: when you change any of the parameters, you have to recompile
c  your programs.
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c
c
c Parameters related to the size of your problem
c ==============================================
c
      INTEGER INTE_ST1
c
c-- NRECMAX = maximum number of elementary records
c-- (Caution: with time-dependent variables this number may be much  
c-- larger than the number of records in your original data file!!)
c
      integer:: NRECMAX=100000
c
c-- NRECMAX2 = maximum number of elementary records in the 'future'
c--    data file (FUTURE statement in PREPARE).
c--    These will normally be much fewer than in the data file
c
!      from Survival Kit v6 onwards just defined here, 
!       the exact value is computed in the program
      integer:: NRECMAX2
c
c-- MXEF = max. number of covariates. These are discrete
c--    (CLASS) covariates and continuous covariates.
c--    Time dependent covariates have to be counted twice (because
c--    the states before and after change occupy two positions on
c--    the recoded data file) !!
c
      integer:: MXEF=30
c
c-- MXEF_USED = max. number of covariates actually used in the
c--    MODEL statement of COX or WEIBULL.
c--    There may be situations where you create recoded files with
c--    very many covariates with PREPARE and never use all of them
c--    together in COX or WEIBULL. In this case it may make sense
c--    to have MXFE_USED smaller that MXEF (it must not be larger!)
c--    to save core memory space.
c
!      from Survival Kit v6 onwards just defined here, 
!       the exact value is computed in the program
      integer:: MXEF_USED
c
c-- MXSTRA= max. number of strata, i.e. levels of the strata variable
c--    defined in the STRATA statement.
c
      integer:: MXSTRA=900
c
c-- NDIMAX = max. total number of levels of effects in the model
c--    (size of the vector of solutions)
c
      integer:: NDIMAX=10000
c        PARAMETER(NDIMAX=2000)
c
c-- NDIMAX2 = this parameter is used to indicate whether the Hessian
c--    matrix is to be stored or not. It is used in COX and (WEIBULL with the
c--    DENSE_HESSIAN statement).
c--    The Hessian is needed to calculate standard deviations of estimated
c--    effects (STD_ERROR statement) and to find dependencies (option FIND
c--    in the CONSTRAINT statement).
c--    In WEIBULL, the Hessian is normally stored in sparse form (see
c--    parameters NZE_HESS (AVAIL and NZE) below) except when stating DENSE_HESSIAN
c--    ndimax2 = ndimax -> Hessian is needed (and fully stored)
c--    ndimax2 = 1      -> Hessian is not needed (or sparsely stored)
c
c--   vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
c--   |  CAUTION:  this is likely to be a limiting parameter for the size  |
c--   |  of your problem. An incorrect specification of this parameter is  |
c--   |  is also one of the most frequent sources of errors when using the |
c--   |  Survival Kit !!                                                   |
c--   vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
c
c       PARAMETER(NDIMAX2=1)
!         PARAMETER(NDIMAX2=NDIMAX)
! specification moved to beginning of main
!   >>> BY DEFAULT: NDIMAX2=NDIMAX -- CHANGABLE WITH KEYWORD: NO_HESSI(AN)
      integer:: NDIMAX2   
c
c-- NTIMMAX = largest possible value of the (dependent) time variable
c--    this value is necessary as an upper limit for an efficient
c--    comutation of log(time) and exp(time) and when calculating
c--    functional values of the survivor curve (SURVIVAL statement
c--    of COX and WEIBULL)
c
      integer:: NTIMMAX=6000
c
c-- NSTIMAX = maximum number of distinct times or quantiles that
c--    can be defined in the SURVIVAL statement options SPECIFIC
c--    and QUANTILE.
c
      integer:: NSTIMAX=100
c-- NCOLPED is the number of columns (-1) for the pedigree file
c-- changable with keyword WITH_MGD
c--    =3 if animal, sire, dam or mgs
c--    =4 if animal, sire, mgs, mgd
!   >>> BY DEFAULT: NCOLPED=3 -- CHANGABLE WITH KEYWORD: WITH_MGD
c
      integer:: NCOLPED=3
c
c-- INTE_ST1 gives the possibility to integrate out one loggamma
c--     random effect with the option INTEGRATE_OUT ... WITHIN ...
c--     WHILE a STRATA statement is specified 
c--     INTE_ST=5 ==> usual case 
c--     INTE_ST=6 ==>  STRATA + INTEGRATE_OUT .. WITHIN ... 
c--     INTE_ST1=7 ==>  STRATA + INTEGRATE_OUT .. WITHIN ...
C                     AND TIME-DEPENDENT STRATA
c
      PARAMETER(INTE_ST1=7)
c
c-- MXLVOUT is the maximum number of levels for the random effect 
c--     which is integrated out ...
c     
      integer:: MXLVOUT=3000000
c
c
c Parameters used in the lbfgs maximisationn routine
c ==================================================
c
      INTEGER ITER_BF
      REAL*8 EPS_BFDEF
c
c-- MVEC_BF = number of vectors used to store the approximation of the Hessian
c--    (rank of approx. Hessian = MVEC_BF). Values between 3 and 20 might be
c--    used).
c
      integer:: MVEC_BF= 15
c
c-- EPS_BSDEF = Default convergence criterion, may be overridden using
c--    the CONVERGENCE_CRITERION statement in COX or WEIBULL.
c--    (||GRADIENT||/FUNCTION < EPS_BS max(1,||BETA||)
c
      PARAMETER(EPS_BFDEF=1.D-8)
c
c-- ITER_BF = maximum number of iterations.
c
       PARAMETER(ITER_BF= 25)
c
c-- NO_LOG relates to the parameter rho of the WEIBULL distribution.
c--    no_log = 1 -> rho is used in the maximisation routine
c--    no_log = 0 -> log(rho) is used in the maximisation routine
c
      integer:: NO_LOG=1
c
c Parameters used in the numerical integration to get moments of the
c marginal posterior of a random effect variance parameter
c (when the MOMENTS option in the RANDOM statement is used)
c===================================================================
c
c-- NPGAUSS = number of points used during Gauss-Hermite integration
c      (has to be between 3 and 5)
c
      integer:: NPGAUSS=5
c
c-- NITER_GAUSS = number of iterations in the Gauss-Hermite integration
c     process
C
      integer:: NITER_GAUSS=5
c
c Parameters relating to sparse storage of the Hessian matrix in WEIBULL
c ======================================================================
c
c-- NDIMSPAR = working vector to store intermediate elements used in
c--    the recoding in PREPARE or during the sparse computation of
c--    the HESSIAN in Weibull
c--    this should be a few times bigger than NDIMAX (no general rule)
c--    (July 2003: obsolete)
c       PARAMETER(NDIMSPAR=NDIMAX)
c
c-- AVAIL = declared dimension of working vector IS (used in fspak.f,
c--    the sparse matrix package of MISZTAL and PEREZ_ENCISO)
c--    the required number is a function of the number of nonzero elements
c--    of the HESSIAN matrix 
c--    (note: this is the dimension of a VECTOR, not a MATRIX) 
c
c
c-- NZE = declared dimension of the vector "HESSIAN" which includes
c--       all nonzero elements of the matrix of second derivatives.
c--       Often, one can take NZE = AVAIL
c--    (note: this is the dimension of a VECTOR, not a MATRIX) 
c
c  NZE, AVAIL and NDIMSPAR substituted with NZE_HESS => nonzero elements 
c                                              in the Hessian matrix
!      PARAMETER(NZE=8000000)
!      PARAMETER(AVAIL=NZE)
!      PARAMETER(NDIMSPAR=NZE)
!       PARAMETER(NZE_HESS=8000000) 
      INTEGER :: NZE_HESS=1000000
c
c Parameters related to the parameter and report files in COX and WEIBULL
c =======================================================================
c
      LOGICAL UPPER_CASE,DETAIL_IT
      LOGICAL SUPPLY_INPUT1,SUPPLY_INPUT2
      CHARACTER*100 TEMPDIR,BACKUP
c
c-- upper_case = .TRUE. if all words (including file names) used in the
      PARAMETER(UPPER_CASE=.false.)
c
c-- SUPPLY_FILE1 : logical variable = .true. if the name of parameter file 1
c--    (provided by PREPARE) is supplied at running time (.false. if supplied
c--    in parinclu)
c
      PARAMETER (SUPPLY_INPUT1=.false.)
c
c-- parm1 : name of parameter file 1 (only if supply_file1=.false.)
c
      CHARACTER*100 :: parm1='varlist.txt'
c
c-- SUPPLY_FILE2 : logical variable = .true. if the name of parameter file 2
c--    is supplied at running time (.false. if supplied in parinclu)
c
      PARAMETER (SUPPLY_INPUT2=.false.)
c
c-- parm2 : name of parameter file 2 (used only if supply_file2=.false.)
c
      CHARACTER*100 :: parm2='weibull.txt'
c
c-- SUPPLY_OUTPUT : logical variable = .true. if the name of the detailed output
c--    (with information about iterations) is supplied at running time
c
      LOGICAL :: SUPPLY_OUTPUT=.true.
c
c-- parmout : name of output file (used only if supply_output=.false.)
c
      CHARACTER*100 :: parmout='logfile.txt'
c
c-- TEMPDIR : name of the directory used for temporary files
c--        (= ' '  if = current directory)
c
      PARAMETER (TEMPDIR=' ')
c
c-- BACKUP  : name of the file used to backup solutions. Solutions
c--    will be stored after each iteration and may be used to restart
c--    the iteration procedure in case the machine breaks down during
c--    the maximisation (you have to use the READ_OLD_SOLUTIONS statement
c--    of COX or WEIBULL then). Only useful for very large applications.
c--    backup = ' ' if no backup file should be produced.
c
      PARAMETER (BACKUP='backup')
c
c-- DETAIL_IT  : switch on (if = .true.) or off (if =.false.) the  
c--    printing of detailed information (for each effect) at each iteration    
c--    of the maximization subroutine (average,mean or max absolute change
c--    between iterations)
      PARAMETER (DETAIL_IT=.false.)
c
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c  PARAMETERS THAT MAY _NOT_ BE MODIFIED BY THE USER
c  Don't you dare look beyond this point !!! :-)
c-----------------------------------------------------------------------
c-----------------------------------------------------------------------
c
c Parameters related to the reading and translation of parameter files
c ====================================================================
c
      INTEGER MAXWORD,MAXLENG,MAXFIG,LRECL,NBUG_2000,NKEYW
c
c-- maxword = max. number of words in the parameter file
c-- maxleng = max. number of characters in a word
c-- maxfig = max. number of figures (i.e. numbers) in the parameter file
c-- lrecl = number of columns read per record
c-- nbug_2000 = maximum year value (yy=00==>nbug_2000) under which a D6 date
c--            is transformed from ddmmyy to ddmm(100+yy) to avoid the 'year 2000 bug'
c-- parameter files should be automatically transformed to upper-case names  
c-- nkeyw = number of keywords used in the parameter file for WEIBULL
      PARAMETER(MAXWORD=1000)
      PARAMETER(MAXLENG=40)
      PARAMETER(MAXFIG=400)
      PARAMETER(LRECL=150)
      PARAMETER(NBUG_2000=40)
      PARAMETER(NKEYW=85)
	  
!*************************************************************
! new part - type definitions and "globalization" of variables
!          previously in common blocks - Meszaros, 2008 - 2009
!*************************************************************

! variables without (allocatable) dimension
!! integers
      integer NTOT,MAXRAND,ITBNEW,NPEST,NRAND
      integer ITOEV,ITOEV2,IOMODE,IR1,IR2
      integer NCOVMOD,NDENS,ISTRVAR,NINTTDEP,NDIMB
      integer NINCLU,NINCLUC,NWITHIN,IWITHCOL,ICHECK,INTEGAM
      integer NSTOR,ICRHO,NDCOV,NSTRATA,NCCOV,NJOINT
      integer NSTR,ITDEPSTR,KCOLPRE,KCOLSTR,KCOLNID	  
      integer KCOLINT,KCOLWITH,FORCE_POS,NRULE,NRELMAT	
      integer NANI,NRELMAT2,nze2,NQUANT,NSTIME,NPR_XNS
! wn (word number) and kn(keyword number) to replace corresponding I-s and J-s
! The same as in prepare.
      integer wn,kn
! flag for user defined output file for LOGFILE
      integer outflag
 !!! two ESTIMATES
      integer NPARMI,MASK(20)
      INTEGER INDXS2(20),KW
      INTEGER NFMAX,NFLAT,NTRACE,NXTRA
 !!!
      REAL*8 XMAX(20),XMIN(20),DELTX(20),DELMIN(20)
      REAL*8 XXS(20)
      REAL*8 FOBJ
   
!! reals
      real TOTIM
      real*8 MU_ADD,OPS,XINCLU 	  
!! characters
      character*100 FILE(12)

!! special cases - with some comments
  
      integer NDIM 
! NDIM used in main, rwsave.f opti.f (funct. FTOMIN) 
! funct.\subr. argument NDIM changed to NDIM1 in sparse2.f, various.f-2x, backsol.f,
!    deriv.f, deriv2.f-2x, init.f, opti.f(in subr. optimize) 	  
	  
      real*8 EPS_BF
! EPS_BF used in predict.f, rwsave.f, main, backsol.f, deriv.f, deriv2.f - 2x, 
!    init.f, opti.f (funct. FTOMIN)
! funct.\subr. argument EPS_BF changed to EPS_BF1 in opti.f(in subr. optimize)

!      real*8 BETA(NDIMAX)
! BETA used in rwsave.f, main, opti.f (funct. FTOMIN)
! funct.\subr. argument BETA changed to BETA1 in predict.f, backsol.f, deriv.f, deriv2.f -2x,
!    init.f, opti.f(in subr. optimize)
! NOW allocatbale
	  
	  
      character*8 KEYWORD(NKEYW)
      integer ik              ! new variable introduced, I changed to ik as i-keyword
      DATA (KEYWORD(ik),ik=1,NKEYW)/'TIME    ','CODE    ','COVARIAT',   ! DATA block moved from init.f
     +'STORAGE ','ON_DISK ','IN_CORE ','STRATA  ','MODEL   ',
     +'RHO_FIXE','ID      ','TEST    ','GENERAL ','EFFECT  ',
     +'SEQUENTI','LAST    ','STD_ERRO','DENSE_HE','INTEGRAT',
     +'JOINT_MO','WITHIN  ','INCLUDE_','PREVIOUS','READ_OLD',
     +'STORE_SO','CONSTRAI','IMPOSE  ','CHECK   ','FIND    ',
     +'LARGEST ','NONE    ','RANDOM  ','LOGGAMMA','NORMAL  ',
     +'MULTINOR','USUAL_RU','MGS_RULE','SIRE_DAM','ESTIMATE',
     +'MOMENTS ','SURVIVAL','QUANTILE','SPECIFIC','EQUAL_SP',
     +'STORE_BI','READ_BIN','STATISTI','ONLY_STA','CONVERGE',
     +'BASELINE','KAPLAN  ','ALL_TIME','JOINCODE','COEFFICI',
     +'FREE_FOR','FIXED_FO','UNFORMAT','BLOCKED_','COMPRESS',
     +'STD     ','FILES   ','DISCRETE','ANIMAL_S','BACK_SOL',
     +'ITE_QUAS','ORIGIN  ','FORCE_PO','PSEUDO_R','NTIMMAX ',
     +'NSTIMAX ','NLEVOUT ','NVEC_BF ','NO_LOGAR','NPGAUSS ',
     +'NITER_GA','NRECMAX ','NEFMAX  ','NSTRAMAX','NDIMAX  ',
     +'NO_HESSI','WITH_MGD','LOGFILE ','NZE_HESS','USE_R   ',
     +'CORRELAT','PRECISIO'/ 

!*************************************************************
!  new parameters in module parinclu
!*************************************************************
! parameter related to variable TAB(21,0:NDIMAX) -> to change the first dimension
      integer tabfirst
      parameter(tabfirst=21)
!*************************************************************	  
!  allocable variables
!*************************************************************

!! integers
      integer, allocatable, dimension(:) :: nfail  !NFAIL(NDIMAX)
      integer, allocatable, dimension(:) :: ianal  !IANAL(MXEF)
!      integer, allocatable, dimension(:) :: ianal1  !IANAL1(MXEF)
!      integer, allocatable, dimension(:) :: ianal2  !IANAL2(MXEF)
      integer, allocatable, dimension(:) :: icons  !ICONS(0:NDIMAX)
      integer, allocatable, dimension(:) :: itest  !ITEST(MXEF)
      integer, allocatable, dimension(:) :: ncoef  !NCOEF(0:MXEF)
      integer, allocatable, dimension(:) ::	icoef  !ICOEF(0:MXEF) 
      integer, allocatable, dimension(:) :: isame  !ISAME(0:MXEF)
      integer, allocatable, dimension(:) :: icovtyp !ICOVTYP(MXEF)
      integer, allocatable, dimension(:) :: ncol    !NCOL(0:MXEF)
      integer, allocatable, dimension(:) :: ifirst  !IFIRST(MXEF)
      integer, allocatable, dimension(:) :: ilast   !ILAST(MXEF)
      integer, allocatable, dimension(:) :: iordr1  !IORDR1(MXEF)
      integer, allocatable, dimension(:) :: iordr2  !IORDR2(MXEF)
      integer, allocatable, dimension(:) :: itrait1 !ITRAIT1(MXEF)
      integer, allocatable, dimension(:) :: itrait2 !ITRAIT2(MXEF)
      integer, allocatable, dimension(:) :: imod    !IMOD(MXEF)
      integer, allocatable, dimension(:) :: maxtim   !MAXTIM(MXSTRA)
      integer, allocatable, dimension(:) :: jmaxtim  !JMAXTIM(MXSTRA)
      integer, allocatable, dimension(:) :: ind      !IND(MXEF)
      integer, allocatable, dimension(:) :: istime   !ISTIME(NSTIMAX)
      integer, allocatable, dimension(:) :: irank    !IRANK(NDIMAX)
	  
      integer, allocatable, dimension(:,:) :: igam     !IGAM(0:MXEF,3)
      integer, allocatable, dimension(:,:) :: inor     !INOR(0:MXEF,3)
      integer, allocatable, dimension(:,:) :: iolcodst !IOLCODST(MXSTRA,3)
      integer, allocatable, dimension(:,:) :: ioldcode !IOLDCODE(NDIMAX,4)
      integer, allocatable, dimension(:,:) :: ipedig   !IPEDIG(NCOLPED+1,NDIMAX)
      integer, allocatable, dimension(:,:) :: ylist    !YLIST(4,NZE_HESS)
	  
!! reals
      real*8, allocatable, dimension(:) :: xgam  !XGAM(0:MXEF)
      real*8, allocatable, dimension(:) :: xnor  !XNOR(0:MXEF)
      real*8, allocatable, dimension(:) :: xav   !XAV(MXEF)
      real*8, allocatable, dimension(:) :: xsq   !XSQ(MXEF)
      real*8, allocatable, dimension(:) :: xcoef !XCOEF(MXEF)
      real*8, allocatable, dimension(:) :: betawoc !BETAWOC(2*MXSTRA+3)
      real*8, allocatable, dimension(:) :: veclog  !VECLOG(NTIMMAX)
      real*8, allocatable, dimension(:) :: valrho  !VALRHO(MXSTRA)
      real*8, allocatable, dimension(:) :: dlogrho !DLOGRHO(MXSTRA)
      real*8, allocatable, dimension(:) :: delta   !DELTA(NDIMAX)
      real*8, allocatable, dimension(:) :: squant  !SQUANT(NSTIMAX)
      real*8, allocatable, dimension(:) :: beta    !BETA(NDIMAX)

      real*8, allocatable, dimension(:,:) :: h       !H(NDIMAX2,NDIMAX2)
      real*8, allocatable, dimension(:,:) :: vecexp  !VECEXP(NTIMMAX,MXSTRA)
      real*8, allocatable, dimension(:,:) :: bound   !BOUND(MXEF,3)

!! characters	  
      character*8, allocatable, dimension(:) :: label !LABEL(0:MXEF)

!! logicals
      logical, allocatable, dimension(:,:) :: tab     !TAB(tabfirst,0:NDIMAX)
	  
! to replace EQUIVALENCE statements - variables from common block DATA
      type blockdef
         integer, dimension (:,:),pointer :: info          !INFO(NRECMAX,INTE_ST1)
         integer, dimension (:,:),pointer :: idata         !IDATA(NRECMAX,MXEF_USED)
         real, dimension(:,:),pointer :: xdata             !XDATA(NRECMAX,MXEF_USED)
      end type blockdef
      type(blockdef) ::blockb
      integer NREC
	  
! to replace EQUIVALENCE statements - variables from common block DATA2
      type blockdef2
         integer, dimension (:,:),pointer :: info2          !INFO2(4,NRECMAX2)
         integer, dimension (:,:),pointer :: idata2         !IDATA2(MXEF_USED,NRECMAX2)
         real, dimension(:,:),pointer :: xdata2             !XDATA2(MXEF_USED,NRECMAX2)
      end type blockdef2
      type(blockdef2) ::blockb2
      integer NREC2

! variable to specify if the R interface should be used
! changeable by keyword USE_R
      logical :: useR=.false.
!
! CORRELATED RANDOM EFFECTS	
!  
! the value of correlation coeficient - initially no correlation assumed
      real*8 :: corr=0.d0
      real*8 :: precorr_mul=1.d0
! flag if the CORRELATION keyword is present - initially no correlation assumed 
!                                              no estimation erquired
      logical :: corrflag=.false.,estcor=.false.
! variables related to the correlated random effects
      integer,allocatable ::  icorr(:,:)   !icorr(MXEF,2)
      real*8 zeta     ! transformed value for correlation (between +- infinity)
      real*8 VM1(2,2) ! inverse or the matrix holding variances and correlation (V-1)
      real*8 detv     ! determinant of the V matrix	 
	  
      end module parinclu	  
