      subroutine SIMUL (seed,n,rho,tmed,tcens,nfix,nlevfix,
     +                  fixmin,fixmax,nsire,variance,nss)
C*********************************************************
C Simulation program to generate simulated data sets for 
C survival analyses
C
C This is a modified version by Gabor Meszaros, 02. March 2010
C  according to the R function requirements
C*********************************************************
C  nbtmt0 : maximum number of levels for each fixed effect
c
      parameter(nbtmt0=500) 
c  n0 : maximum number of records simulated
c  nsire0 : maximum number of sires
c  nss0 : maximum number of sires of sires
      PARAMETER (nsire0=500,N0=10000,nss0=100,nef=3)
      INTEGER I,J,icens,ntmt(nbtmt0,nef) 
      INTEGER itmt(nef),seed,n,tcens,nfix
      REAL*8 std,s,x1,x2,Z,Y,U,RHO,lamb,X,SUM,TMT(nbtmt0,nef),lamb0
      REAL*8 sire(nsire0),TIME,variance,xnbtmt,SS(NSS0),vec(nbtmt0)
      REAL*8 std14,std34,std3,tmed,yfix(nef)
      INTEGER nlevfix(*)	  
      REAL*8 fixmin(*),fixmax(*)
c
!         **********************************
!         *   SIMULATION OF WEIBULL DATA   *
!         **********************************
!         This program will generate a data set "datsim" with
!         N (possibly censored) records from a Weibull distribution.
!         Up to 3 fixed effects and one random (normal) "sire" effect
!         are allowed. A simple relationship matrix (through sires of sires)
!         can be included. True parameter values are
!         stored in file "true.txt".

! setting array to zero
       do i=1, nbtmt0
          do j=1, nef
            ntmt(i,j)=0
          enddo
       enddo
C*********************************************************
      ifail= 0
      open(8,file='true.txt',form='formatted')
c---------------------------------------------------------
c seed to be used in the random number generation process
!      write(*,*) ' write any integer number, to use as seed',
!     +    ' for random number generation:'
!      read(*,*) seed
!      If (seed.eq.0) stop 
c---------------------------------------------------------
c number of records to simulate
!      write(*,*) ' choose the number of records to simulate (maximum',
!     +   N0,'):'
!      read(*,*) n
c---------------------------------------------------------
c weibull parameters rho and rho * log lambda
!      write(*,*) ' value of the Weibull (rho) parameter: ' 
!      write(*,*) ' rho = 1: exponential; rho > 1: increasing hazard;',
!     +    ' rho < 1 : decreasing hazard'
      write(8,*) ' value of the Weibull (rho) parameter: ' 
!      read(*,*) rho
      write(8,*) rho
!      if (rho.le.0.d0) stop
!      write(*,*)
!     + ' Median value of failure time (in absence of censoring):' 
      write(8,*)
     + ' Median value of failure time (in absence of censoring):' 
!      read(*,*) tmed
      write(8,*) tmed
!      if (tmed.le.0.d0) stop
      x = dlog(-dlog(0.5d0))-rho*dlog(tmed)
      lamb0 =  x
      write(8,*) ' value of the Weibull (rho * log lambda) parameter'
      write(8,*) lamb0 
c---------------------------------------------------------
c number of fixed effects
!      write(*,*) ' choose the number of fixed effects (1 to 3):'
!      read(*,*) nfix
!      if (nfix.lt.1.or.nfix.gt.3) stop
c---------------------------------------------------------
c for each fixed effect: number of levels, and then x and xm  such that
c the effect is generated from a ]log x, + log xm[  Uniform distribution
      DO 10 ifix = 1, nfix 
!      write(*,*) ' choose the number of levels for effect',ifix,
!     +  ' (maximum =',nbtmt0,'):'
!      read(*,*) nlevfix(ifix)
!      write(*,*) ' choose the smallest and the largest ',
!     +  'relative risks' 
!      write(*,*) ' that can be generated for',
!     +   ' this effect (e.g., 0.7 1.3)'
!      read(*,*) x,xm
!      if (xm.lt.x) then
!        y =xm
!        xm= x
!        x = y
!      endif
!      if (x.lt.0.) then 
!       write(*,*)' the smallest relative risk should be larger than 0'
!       stop
!      endif
      fixmin(ifix)=log(fixmin(ifix)) 
      fixmax(ifix)=log(fixmax(ifix))
      yfix(ifix)= fixmax(ifix)- fixmin(ifix) 
      IF (yfix(ifix).gt.0) call uniran(nlevfix(ifix),seed,vec)
c      IF (y.gt.0) call G05FAF(0,y,nlevfix(ifix),vec)
      do 8 i=1,nlevfix(ifix)
        vec(i)=vec(i)*yfix(ifix)
        tmt(i,ifix) = vec(i) + fixmin(ifix) 
        z =dexp(tmt(i,ifix)) 
!        write(*,103) ifix,i,tmt(i,ifix),z
        write(8,103) ifix,i,tmt(i,ifix),z
 103    format(1x,'fixed  effect ',I2,' : level ',i4,' beta= ',
     +    F8.4,5x,' risk = ',f8.4)
    8 continue
  10  continue
c---------------------------------------------------------
c random effect
!      write(*,*) ' Do you want to include a random effect (Y/N)?'
!      read(*,*) ans
!      nsire=0
!      if (ans.eq.'n'.or.ans.eq.'N') goto 50
      if (nsire.ne.0) then  ! *mark
c---------------------------------------------------------
c number of sires
!      write(*,*) ' choose the number of random effects (sires?) ',
!     +   'to simulate (maximum',nsire0,'):'
!      read(*,*) nsire
c---------------------------------------------------------
c variance of the random effect (normal distribution assumed)
!      write(*,*) ' true variance of the normal distribution used',
!     +    ' for the random effect:' 
!      write(*,*) '    (a value > 0.1 does not make much sense ...):'
!      read(*,*) variance 
      std = dsqrt(variance) 
      std14 = dsqrt(variance/4.d0)
      std34 =dsqrt(variance*3.d0/4.d0)  
      std3 =dsqrt(variance*3.d0)  
      xmean=0.d0
      x2=0.d0
      k=0
c---------------------------------------------------------
c possibility to include relationships between sires through 
c sires of sires
!      write(*,*) ' Do you want the random effect to represent',
!     +   ' related sires (Y/N)?'
!      read(*,*) ans
!      nss=0
!      if (ans.ne.'n'.and.ans.ne.'N') then 
      if (nss.ne.0) then ! **mark
!      write(*,*) ' choose the number of sires of sires',
!     + ' to simulate (maximum',Nss0,'):'
!      read(*,*) nss
      nss=min(nss,nsire)
c---------------------------------------------------------
c pedigs = pedigree file
c first generate sires of sires effects
      OPEN(2,file='pedigs.txt')
      DO 400 IP=1,NSS
          IC=1
          js=nsire+ IP
          call norran(1,seed,s)
          SS(IP)=s* STD14 
c          SS(IP)=G05DDF(0.D0,STD14) 
          WRITE(2,101) js,ic,0,0
 101      FORMaT(1X,4I8)
          J1 = nsire/nss 
c---------------------------------------------------------
c generate the sire effects         
          do 410  J=1+(IP-1)*J1,IP*J1
            call norran(1,seed,s)
            z= s * STD14 
c            Z=G05DDF(0.D0,STD34) 
            k=k+1
            sire(J) = SS(IP) + Z
            z = dexp(sire(j))
            write(8,106) j,sire(j),z
106         format(1x,'sire ',I5,' ==> effect= ',F8.4,5x,
     +          ' risk ratio= ',f8.4)
            IC=2
            jp = j 
            write(2,101) jp,ic,js,0
410       continue
400   CONTINUE
      else ! **mark
c---------------------------------------------------------
c generate the sire effects 
        do 420 I=1,nsire
            call norran(1,seed,s)
            z= s * STD 
c            Z=G05DDF(0.D0,std) 
            sire(i)= z
 420    continue  
      endif ! **mark
C
c---------------------------------------------------------
c calculation of the mean and variance of the sire effect
      DO 20 I=1,nsire
        xmean=xmean+sire(I)
        X2=X2 + sire(I)*sire(I)
  20  CONTINUE
      xmean= xmean/DBLE(nsire)
      X2= (X2 - nsire* xmean * xmean)/(nsire-1.d0)
!      write(*,*) 'mean of the simulated random effects= ',xmean
      WRITE(8,*) 'mean of the simulated random effects= ',xmean
!      write(*,*) '         variance=',X2
      WRITE(8,*) '         variance=',X2
c
!  50  continue 
      endif ! *mark
c      lamb0=DEXP(lamb0)
c---------------------------------------------------------
c specification of the censoring time (same for all records)
!      write(*,*) ' censoring time (0 or a large value = no censoring) ?'
!      read(*,*) tcens 
!      tcens=max(0,tcens) 
!      if (tcens.eq.0) tcens=1000000
      SUM=0.0
      IUN=1
c---------------------------------------------------------
c datsim = data file
      OPEN(1,file='datsim.txt')
      DO 500 I=1,N 
c---------------------------------------------------------
c sire number = mod(i,nsire)
         if (nsire.ne.0) then
            js=MOD(I,nsire)+1
c            ds= dexp(-sire(js))
            ds= sire(js)
         else 
            ds =0.0
         endif
         dx = 0.d0
c---------------------------------------------------------
c fixed effect(s) level : based on a uniform distribution
c from 1 to number of levels
         do 510 jj=1,nfix          
           XNBTMT=nlevfix(jj)
           call uniran(1,seed,Z)
           z= 1.D0 + xnbtmt * z
c           CaLL G05FaF(1.D0,XNBTMT,1,Z)
           itmt(jj) = INT(Z)
           ntmt(itmt(jj),jj) = ntmt(itmt(jj),jj) + 1
           dx = dx + tmt(itmt(jj),jj)
  510    continue 
c---------------------------------------------------------
c generation of the Weibull record
         lamb = lamb0 + dx + ds 
c         IF (lamb.le.0) write(*,*) dx,ds,lamb0,
c     +         (tmt(itmt(jj),k),k=1,nfix)
c         X= G05DPF(RHO,lamb,IFAIL)
         call weiranm(1,seed,RHO,LAMB,X)
c---------------------------------------------------------
c perform censoring if relevant (censoring point = tcens)
         ICENS=1
         NCENS=0
          if (tcens.ne.0) then
           IF (x.gt.tcens) THEN
              x =tcens 
              ICENS=0
           ENDIF
          endif
         IF (ICENS.EQ.0) then
            NCENS = NCENS+1
         endif
         long= MaX(1,int(x+0.5))
         SUM = SUM + long
         IC=0
c---------------------------------------------------------
c write on data file
         if (nsire.eq.0) then
           write(1,100) I,LONG,ICENS,(itmt(k),k=1,nfix)
         else 
           write(1,100) I,LONG,ICENS,(itmt(k),k=1,nfix),js
         endif
  500 CONTINUE
  100 FORMaT(1X,7I9)
      CLOSE(1)
c---------------------------------------------------------
c statistics
      SUM = SUM/DBLE(N)
      z = 100.* dble(ncens)/dble(n)
!      write(*,98) sum,ncens,z
      write(8,98) sum,ncens,z
   98 format(/1x,'mean longevity  =',f8.2,'  censored=',i5,
     +      ' = ',f5.2,'%')
      do 102 jj= 1,nfix
!       write(*,*) 'number of observations for effect ',jj,' :'
       write(8,*) 'number of observations for effect ',jj,' :'
       do 107 i=1,nlevfix(jj)
 !        write(*,104) I,ntmt(I,jj)
         write(8,104) I,ntmt(I,jj)
 104    format(1x,' level ',i4,' :',I5)
 107   continue
 102  continue
      close(8)
      END 
	  
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C   END OF THE MAIN PART
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
 
      REAL*8 FUNCTION GAMLOG(XVALUE)
C
C     ALGORITHM AS245  APPL. STATIST. (1989) VOL. 38, NO. 2
C
C     Calculation of the logarithm of the gamma function
C
      IMPLICIT NONE
      INTEGER IFAULT
      REAL*8 ALR2PI, FOUR, HALF, ONE, ONEP5, R1(9), R2(9),
     +          R3(9), R4(5), TWELVE, X, X1, X2, XLGE, XLGST, XVALUE,
     +          Y, ZERO
C
C     Coefficients of rational functions
C
      DATA R1/-2.66685 51149 5D0, -2.44387 53423 7D1,
     +        -2.19698 95892 8D1,  1.11667 54126 2D1,
     +         3.13060 54762 3D0,  6.07771 38777 1D-1,
     +         1.19400 90572 1D1,  3.14690 11574 9D1,
     +         1.52346 87407 0D1/
      DATA R2/-7.83359 29944 9D1, -1.42046 29668 8D2,
     +         1.37519 41641 6D2,  7.86994 92415 4D1,
     +         4.16438 92222 8D0,  4.70668 76606 0D1,
     +         3.13399 21589 4D2,  2.63505 07472 1D2,
     +         4.33400 02251 4D1/
      DATA R3/-2.12159 57232 3D5,  2.30661 51061 6D5,
     +         2.74647 64470 5D4, -4.02621 11997 5D4,
     +        -2.29660 72978 0D3, -1.16328 49500 4D5,
     +        -1.46025 93751 1D5, -2.42357 40962 9D4,
     +        -5.70691 00932 4D2/
      DATA R4/ 2.79195 31791 8525D-1, 4.91731 76105 05968D-1,
     +         6.92910 59929 1889D-2, 3.35034 38150 22304D0,
     +         6.01245 92597 64103D0/
C
C     Fixed constants
C
      DATA ALR2PI/9.18938 53320 4673D-1/, FOUR/4.D0/, HALF/0.5D0/,
     +     ONE/1.D0/, ONEP5/1.5D0/, TWELVE/12.D0/, ZERO/0.D0/
C
C     Machine-dependant constants.
C     A table of values is given at the top of page 399 of the paper.
C     These values are for the IEEE double-precision format for which
C     B = 2, t = 53 and U = 1023 in the notation of the paper.
C
      DATA XLGE/5.10D6/, XLGST/1.D+305/
C
      X = XVALUE
      GAMLOG = ZERO
C
C     Test for valid function argument
C
      IFAULT = 2
      IF (X .GE. XLGST) GOTO 99 
      IFAULT = 1
      IF (X .LE. ZERO) GOTO 99 
      IFAULT = 0
C
C     Calculation for 0 < X < 0.5 and 0.5 <= X < 1.5 combined
C
      IF (X .LT. ONEP5) THEN
        IF (X .LT. HALF) THEN
          GAMLOG = -LOG(X)
          Y = X + ONE
C
C     Test whether X < machine epsilon
C
          IF (Y .EQ. ONE) RETURN
        ELSE
          GAMLOG = ZERO
          Y = X
          X = (X - HALF) - HALF
        END IF
        GAMLOG = GAMLOG + X * ((((R1(5)*Y + R1(4))*Y + R1(3))*Y
     +                + R1(2))*Y + R1(1)) / ((((Y + R1(9))*Y + R1(8))*Y
     +                + R1(7))*Y + R1(6))
        RETURN
      END IF
C
C     Calculation for 1.5 <= X < 4.0
C
      IF (X .LT. FOUR) THEN
        Y = (X - ONE) - ONE
        GAMLOG = Y * ((((R2(5)*X + R2(4))*X + R2(3))*X + R2(2))*X
     +              + R2(1)) / ((((X + R2(9))*X + R2(8))*X + R2(7))*X
     +              + R2(6))
        RETURN
      END IF
C
C     Calculation for 4.0 <= X < 12.0
C
      IF (X .LT. TWELVE) THEN
        GAMLOG = ((((R3(5)*X + R3(4))*X + R3(3))*X + R3(2))*X + R3(1)) /
     +            ((((X + R3(9))*X + R3(8))*X + R3(7))*X + R3(6))
        RETURN
      END IF
C
C     Calculation for X >= 12.0
C
      Y = LOG(X)
      GAMLOG = X * (Y - ONE) - HALF * Y + ALR2PI
      IF (X .GT. XLGE) RETURN
      X1 = ONE / X
      X2 = X1 * X1
      GAMLOG = GAMLOG + X1 * ((R4(3)*X2 + R4(2))*X2 + R4(1)) /
     +              ((X2 + R4(5))*X2 + R4(4))
      RETURN
  99  write(*,100) IFAULT,X
 100  FORMAT(1X,'PROBLEM WITH FUNCTION GAMLOG ',
     +   /1X,'IFAULT =',I3,' X =',G12.5)
      STOP
      RETURN     
      END
* ======================================================================
* NIST Guide to Available Math Software.
* Fullsource for module NORRAN from package DATAPAC.
* Retrieved from CAMSUN on Fri Apr  4 07:09:27 1997.
* ======================================================================
* NORRAN
      SUBROUTINE NORRAN(N,ISEED,X)
      IMPLICIT DOUBLE PRECISION (A-H,P-Z)
C
C     PURPOSE--THIS SUBROUTINE GENERATES A RANDOM SAMPLE OF SIZE N
C              FROM THE THE NORMAL (GAUSSIAN)
C              DISTRIBUTION WITH MEAN = 0 AND STANDARD DEVIATION = 1.
C              THIS DISTRIBUTION IS DEFINED FOR ALL X AND HAS
C              THE PROBABILITY DENSITY FUNCTION
C              F(X) = (1/SQRT(2*PI))*EXP(-X*X/2).
C     INPUT  ARGUMENTS--N      = THE DESIRED INTEGER NUMBER
C                                OF RANDOM NUMBERS TO BE
C                                GENERATED.
C     OUTPUT ARGUMENTS--X      = A DOUBLE PRECISION VECTOR
C                                (OF DIMENSION AT LEAST N)
C                                INTO WHICH THE GENERATED
C                                RANDOM SAMPLE WILL BE PLACED.
C     OUTPUT--A RANDOM SAMPLE OF SIZE N
C             FROM THE NORMAL DISTRIBUTION
C             WITH MEAN = 0 AND STANDARD DEVIATION = 1.
C     PRINTING--NONE UNLESS AN INPUT ARGUMENT ERROR CONDITION EXISTS.
C     RESTRICTIONS--THERE IS NO RESTRICTION ON THE MAXIMUM VALUE
C                   OF N FOR THIS SUBROUTINE.
C     OTHER DATAPAC   SUBROUTINES NEEDED--UNIRAN.
C     FORTRAN LIBRARY SUBROUTINES NEEDED--LOG, SQRT, SIN, COS.
C     MODE OF INTERNAL OPERATIONS--DOUBLE PRECISION.
C     LANGUAGE--ANSI FORTRAN (1977)
C     METHOD--BOX-MULLER ALGORITHM.
C     REFERENCES--BOX AND MULLER, 'A NOTE ON THE GENERATION
C                 OF RANDOM NORMAL DEVIATES', JOURNAL OF THE
C                 ASSOCIATION FOR COMPUTING MACHINERY, 1958,
C                 PAGES 610-611.
C               --TOCHER, THE ART OF SIMULATION,
C                 1963, PAGES 33-34.
C               --HAMMERSLEY AND HANDSCOMB, MONTE CARLO METHODS,
C                 1964, PAGE 39.
C               --JOHNSON AND KOTZ, CONTINUOUS UNIVARIATE
C                 DISTRIBUTIONS--1, 1970, PAGES 40-111.
C     WRITTEN BY--JAMES J. FILLIBEN
C                 STATISTICAL ENGINEERING DIVISION
C                 CENTER FOR APPLIED MATHEMATICS
C                 NATIONAL BUREAU OF STANDARDS
C                 WASHINGTON, D. C. 20234
C                 PHONE--301-921-3651
C     NOTE--DATAPLOT IS A REGISTERED TRADEMARK
C           OF THE NATIONAL BUREAU OF STANDARDS.
C           THIS SUBROUTINE MAY NOT BE COPIED, EXTRACTED,
C           MODIFIED, OR OTHERWISE USED IN A CONTEXT
C           OUTSIDE OF THE DATAPLOT LANGUAGE/SYSTEM.
C     LANGUAGE--ANSI FORTRAN (1966)
C               EXCEPTION--HOLLERITH STRINGS IN FORMAT STATEMENTS
C                          DENOTED BY QUOTES RATHER THAN NH.
C     VERSION NUMBER--82.6
C     ORIGINAL VERSION--JUNE      1972.
C     UPDATED         --SEPTEMBER 1975.
C     UPDATED         --NOVEMBER  1975.
C     UPDATED         --JULY      1976.
C     UPDATED         --DECEMBER  1981.
C     UPDATED         --MAY       1982.
C
C-----CHARACTER STATEMENTS FOR NON-COMMON VARIABLES-------------------
C
C---------------------------------------------------------------------
C
      DIMENSION X(*)
      DIMENSION Y(2)
C
C---------------------------------------------------------------------
C
CCCCC CHARACTER*4 IFEEDB
CCCCC CHARACTER*4 IPRINT
C
CCCCC COMMON /MACH/IRD,IPR,CPUMIN,CPUMAX,NUMBPC,NUMCPW,NUMBPW
CCCCC COMMON /PRINT/IFEEDB,IPRINT
C
      IPR=6
C
C-----DATA STATEMENTS-------------------------------------------------
C
      DATA PI/3.14159265359/
C
C-----START POINT-----------------------------------------------------
C
C     CHECK THE INPUT ARGUMENTS FOR ERRORS
C
      IF(N.LT.1)GOTO50
      GOTO90
   50 WRITE(IPR, 5)
      WRITE(IPR,47)N
      RETURN
   90 CONTINUE
    5 FORMAT(1H , 91H***** FATAL ERROR--THE FIRST  INPUT ARGUMENT TO THE
     1 NORRAN SUBROUTINE IS NON-POSITIVE *****)
   47 FORMAT(1H , 35H***** THE VALUE OF THE ARGUMENT IS ,I8   ,6H *****)
C
C     GENERATE N UNIFORM (0,1) RANDOM NUMBERS;
C     THEN GENERATE 2 ADDITIONAL UNIFORM (0,1) RANDOM NUMBERS
C     (TO BE USED BELOW IN FORMING THE N-TH NORMAL
C     RANDOM NUMBER WHEN THE DESIRED SAMPLE SIZE N
C     HAPPENS TO BE ODD).
C
      CALL UNIRAN(N,ISEED,X)
      CALL UNIRAN(2,ISEED,Y)
C
C     GENERATE N NORMAL RANDOM NUMBERS
C     USING THE BOX-MULLER METHOD.
C
      DO200I=1,N,2
      IP1=I+1
      U1=X(I)
      IF(I.EQ.N)GOTO210
      U2=X(IP1)
      GOTO220
  210 U2=Y(2)
  220 ARG1=-2.0*LOG(U1)
      ARG2=2.0*PI*U2
      SQRT1=SQRT(ARG1)
      Z1=SQRT1*COS(ARG2)
      Z2=SQRT1*SIN(ARG2)
      X(I)=Z1
      IF(I.EQ.N)GOTO200
      X(IP1)=Z2
  200 CONTINUE
C
      RETURN
      END
* WEIRANM
      SUBROUTINE WEIRANM(N,ISEED,RHO,XLAMB,X)
      IMPLICIT DOUBLE PRECISION (A-H,P-Z)
C = MODIFIED VERSION OF WEINRAN (TO INCLUDE XLAMB)
C
C     PURPOSE--THIS SUBROUTINE GENERATES A RANDOM SAMPLE OF SIZE N
C              FROM THE WEIBULL DISTRIBUTION
C              WITH TAIL LENGTH PARAMETER VALUE = RHO.
C              THE PROTOTYPE WEIBULL DISTRIBUTION USED
C              HEREIN IS DEFINED FOR ALL POSITIVE X,
C              AND HAS THE PROBABILITY DENSITY FUNCTION
C  F(X) = XLAMB* RHO * ((XLAMB*X)**(RHO-1)) * EXP(-((XLAMB*X)**RHO)).
C     INPUT  ARGUMENTS--N      = THE DESIRED INTEGER NUMBER
C                                OF RANDOM NUMBERS TO BE
C                                GENERATED.
C                     --RHO  = THE DOUBLE PRECISION VALUE OF THE
C                                TAIL LENGTH PARAMETER.
C                                RHO SHOULD BE POSITIVE
C                     -- XLAMB = SCALE PARAMETER .
C     OUTPUT ARGUMENTS--X      = A DOUBLE PRECISION VECTOR
C                                (OF DIMENSION AT LEAST N)
C                                INTO WHICH THE GENERATED
C                                RANDOM SAMPLE WILL BE PLACED.
C     OUTPUT--A RANDOM SAMPLE OF SIZE N
C             FROM THE WEIBULL DISTRIBUTION
C             WITH TAIL LENGTH PARAMETER VALUE = RHO.
C     PRINTING--NONE UNLESS AN INPUT ARGUMENT ERROR CONDITION EXISTS.
C     RESTRICTIONS--THERE IS NO RESTRICTION ON THE MAXIMUM VALUE
C                   OF N FOR THIS SUBROUTINE.
C                 --RHO SHOULD BE POSITIVE.
C     OTHER DATAPAC   SUBROUTINES NEEDED--UNIRAN.
C     FORTRAN LIBRARY SUBROUTINES NEEDED--LOG.
C     MODE OF INTERNAL OPERATIONS--DOUBLE PRECISION.
C     LANGUAGE--ANSI FORTRAN (1977)
C     REFERENCES--TOCHER, THE ART OF SIMULATION,
C                 1963, PAGES 14-15.
C               --HAMMERSLEY AND HANDSCOMB, MONTE CARLO METHODS,
C                 1964, PAGE 36.
C               --JOHNSON AND KOTZ, CONTINUOUS UNIVARIATE
C                 DISTRIBUTIONS--1, 1970, PAGES 250-271.
C               --HASTINGS AND PEACOCK, STATISTICAL
C                 DISTRIBUTIONS--A HANDBOOK FOR
C                 STUDENTS AND PRACTITIONERS, 1975,
C                 PAGE 128.
C     WRITTEN BY--JAMES J. FILLIBEN
C                 STATISTICAL ENGINEERING DIVISION
C                 CENTER FOR APPLIED MATHEMATICS
C                 NATIONAL BUREAU OF STANDARDS
C                 WASHINGTON, D. C. 20234
C                 PHONE--301-921-3651
C     NOTE--DATAPLOT IS A REGISTERED TRADEMARK
C           OF THE NATIONAL BUREAU OF STANDARDS.
C           THIS SUBROUTINE MAY NOT BE COPIED, EXTRACTED,
C           MODIFIED, OR OTHERWISE USED IN A CONTEXT
C           OUTSIDE OF THE DATAPLOT LANGUAGE/SYSTEM.
C     LANGUAGE--ANSI FORTRAN (1966)
C               EXCEPTION--HOLLERITH STRINGS IN FORMAT STATEMENTS
C                          DENOTED BY QUOTES RATHER THAN NH.
C     VERSION NUMBER--82.6
C     ORIGINAL VERSION--NOVEMBER  1975.
C     UPDATED         --DECEMBER  1981.
C     UPDATED         --MAY       1982.
C
C-----CHARACTER STATEMENTS FOR NON-COMMON VARIABLES-------------------
C
C---------------------------------------------------------------------
C
      DIMENSION X(*)
C
C---------------------------------------------------------------------
C
CCCCC CHARACTER*4 IFEEDB
CCCCC CHARACTER*4 IPRINT
C
CCCCC COMMON /MACH/IRD,IPR,CPUMIN,CPUMAX,NUMBPC,NUMCPW,NUMBPW
CCCCC COMMON /PRINT/IFEEDB,IPRINT
C
      IPR=6
C
C-----START POINT-----------------------------------------------------
C
C     CHECK THE INPUT ARGUMENTS FOR ERRORS
C
      IF(N.LT.1)GOTO50
      IF(RHO.LE.0.0)GOTO60
      GOTO90
   50 WRITE(IPR, 5)
      WRITE(IPR,47)N
      RETURN
   60 WRITE(IPR,15)
      WRITE(IPR,46)RHO
      RETURN
   90 CONTINUE
    5 FORMAT(1H , 91H***** FATAL ERROR--THE FIRST  INPUT ARGUMENT TO THE
     1 WEIRAN SUBROUTINE IS NON-POSITIVE *****)
   15 FORMAT(1H , 91H***** FATAL ERROR--THE SECOND INPUT ARGUMENT TO THE
     1 WEIRAN SUBROUTINE IS NON-POSITIVE *****)
   46 FORMAT(1H , 35H***** THE VALUE OF THE ARGUMENT IS ,E15.8,6H *****)
   47 FORMAT(1H , 35H***** THE VALUE OF THE ARGUMENT IS ,I8   ,6H *****)
C
C     GENERATE N UNIFORM (0,1) RANDOM NUMBERS;
C
      CALL UNIRAN(N,ISEED,X)
C
C     GENERATE N WEIBULL DISTRIBUTION RANDOM NUMBERS
C     USING THE PERCENT POINT FUNCTION TRANSFORMATION METHOD.
C
      DO100I=1,N
      X(I)=(-LOG(1.0-X(I)))/DEXP(XLAMB)
      X(I)=X(I)**(1.0/RHO)
  100 CONTINUE
C
      RETURN
      END
* UNIRAN
      SUBROUTINE UNIRAN(N,ISEED,X)
      IMPLICIT DOUBLE PRECISION (A-H,P-Z)
C
C     PURPOSE--THIS SUBROUTINE GENERATES A RANDOM SAMPLE OF SIZE N
C              FROM THE UNIFORM (RECTANGULAR)
C              DISTRIBUTION ON THE UNIT INTERVAL (0,1).
C              THIS DISTRIBUTION HAS MEAN = 0.5
C              AND STANDARD DEVIATION = SQRT(1/12) = 0.28867513.
C              THIS DISTRIBUTION HAS THE PROBABILITY
C              DENSITY FUNCTION F(X) = 1.
C
C     INPUT  ARGUMENTS--N      = THE DESIRED INTEGER NUMBER 
C                                OF RANDOM NUMBERS TO BE
C                                GENERATED.
C                     --ISEED  = AN INTEGER ISEED VALUE
C     OUTPUT ARGUMENTS--X      = A DOUBLE PRECISION VECTOR
C                                (OF DIMENSION AT LEAST N)
C                                INTO WHICH THE GENERATED
C                                RANDOM SAMPLE WILL BE PLACED.
C     OUTPUT--A RANDOM SAMPLE OF SIZE N 
C             FROM THE RECTANGULAR DISTRIBUTION ON (0,1).
C     PRINTING--NONE UNLESS AN INPUT ARGUMENT ERROR CONDITION EXISTS. 
C     RESTRICTIONS--THERE IS NO RESTRICTION ON THE MAXIMUM VALUE
C                   OF N FOR THIS SUBROUTINE.
C     OTHER           SUBROUTINES NEEDED--NONE.
C     FORTRAN LIBRARY SUBROUTINES NEEDED--NONE.
C     MODE OF INTERNAL OPERATIONS--DOUBLE PRECISION.
C     LANGUAGE--ANSI FORTRAN (1977)
C
C     ALGORITHM--FIBONACCI GENERATOR
C                AS DEFINED BY GEORGE MARSAGLIA.
C
C     NOTE--THIS GENERATOR IS TRANSPORTABLE.
C           IT IS NOT MACHINE-INDEPENDENT
C           IN THE SENSE THAT FOR A GIVEN VALUE
C           OF THE INPUT SEED ISEED AND FOR A GIVEN VALUE
C           OF MDIG (TO BE DEFINED BELOW),
C           THE SAME SEQUENCE OF UNIRFORM RANDOM
C           NUMBERS WILL RESULT ON DIFFERENT COMPUTERS
C           (VAX, PRIME, PERKIN-ELMER, IBM, UNIVAC, HONEYWELL, ETC.)
C
C     NOTE--IF MDIG = 32 AND IF ISEED = 305,
C           THEN THE OUTPUT FROM THIS GENERATOR SHOULD BE AS FOLLOWS--
C           THE FIRST      NUMBER TO RESULT IS .4771580...
C           THE SECOND     NUMBER TO RESULT IS .4219293...
C           THE THIRD      NUMBER TO RESULT IS .6646181...
C           ...
C           THE THOUSANDTH NUMBER TO RESULT IS .2036834...
C
C     NOTE--IF MDIG = 16 AND IF ISEED = 305,
C           THEN THE OUTPUT FROM THIS GENERATOR SHOULD BE AS FOLLOWS--
C           THE FIRST      NUMBER TO RESULT IS .027832881...
C           THE SECOND     NUMBER TO RESULT IS .56102176... 
C           THE THIRD      NUMBER TO RESULT IS .41456343... 
C           ...
C           THE THOUSANDTH NUMBER TO RESULT IS .19797357... 
C
C     NOTE--IT IS RECOMMENDED THAT UPON 
C           IMPLEMENTATION OF DATAPLOT, THE OUTPUT
C           FROM UNIRAN BE CHECKED FOR AGREEMENT
C           WITH THE ABOVE SAMPLE OUTPUT.
C           ALSO, THERE ARE MANY ANALYSIS AND DIAGNOSTIC
C           TOOLS IN DATAPLOT THAT WILL ALLOW THE 
C           TESTING OF THE RANDOMNESS AND UNIFORMITY
C           OF THIS GENERATOR.
C           SUCH CHECKING IS ESPECIALLY IMPORTANT 
C           IN LIGHT OF THE FACT THAT OTHER DATAPLOT RANDOM 
C           NUMBER GENERATOR SUBROUTINES (NORRAN--NORMAL,
C           LOGRAN--LOGISTIC, ETC.) ALL MAKE USE OF INTERMEDIATE
C           OUTPUT FROM UNIRAN.
C
C     NOTE--THE OUTPUT FROM THIS SUBROUTINE DEPENDS
C           ON THE INPUT SEED (ISEED) AND ON THE
C           VALUE OF MDIG.
C           MDIG MAY NOT BE SMALLER THAN 16.
C           MDIG MAY NOT BE LARGER THAN MAX INTEGER ON YOUR COMPUTER. 
C
C     NOTE--BECAUSE OF THE PREPONDERANCE OF MAINFRAMES
C           WHICH HAVE WORDS OF 32 BITS AND LARGER
C           (E.G, VAX (= 32 BITS), UNIVAC (= 36 BITS), CDC (= 60 BITS), ETC.)
C           MDIG HAS BEEN SET TO 32.
C           THUS THE SAME SEQUENCE OF RANDOM NUMBERS SHOULD RESULT
C           ON ALL OF THESE COMPUTERS.
C
C     NOTE--FOR SMALLER WORD SIZE COMPUTERS (E.G., 24-BIT AND 16-BIT),
C           THE VALUE OF MDIG SHOULD BE CHANGED TO 24 OR 16.
C           IN SUCH CASE, THE OUTPUT WILL NOT BE IDENTICAL TO
C           THE OUTPUT WHEN MDIG = 32.
C
C     NOTE--THE CYCLE OF THE RANDOM NUMBERS DEPENDS ON MDIG.
C           THE CYCLE FROM MDIG = 32 IS LONG ENOUGH FOR MOST
C           PRACTICAL APPLICATIONS.
C           IF A LONGER CYCLE IS DESIRED, THEN INCREASE MDIG.
C
C     NOTE--THE SEED MAY BE ANY POSITIVE INTEGER. 
C           NO APPRECIABLE DIFFERENCE IN THE QUALITY
C           OF THE RANDOM NUMBERS HAS BEEN NOTED
C           BY THE CHOICE OF THE SEED.  THERE IS NO
C           NEED TO USE PRIMES, NOR TO USE EXCEPTIONALLY
C           LARGE NUMBERS, ETC.
C
C     REFERENCES--MARSAGLIA G., "COMMENTS ON THE PERFECT UNIFORM RANDOM
C                 NUMBER GENERATOR", UNPUBLISHED NOTES, WASH S. U.
C               --JOHNSON AND KOTZ, CONTINUOUS UNIVARIATE
C                 DISTRIBUTIONS--2, 1970, PAGES 57-74.
C     WRITTEN BY--JAMES BLUE
C                 SCIENTIFIC COMPUTING DIVISION
C                 CENTER FOR APPLIED MATHEMATICS
C                 NATIONAL BUREAU OF STANDARDS
C                 WASHINGTON, D. C. 20234
C               --DAVID KAHANER
C                 SCIENTIFIC COMPUTING DIVISION
C                 CENTER FOR APPLIED MATHEMATICS
C                 NATIONAL BUREAU OF STANDARDS
C               --GEORGE MARSAGLIA
C                 COMPUTER SCIENCE DEPARTMENT
C                 WASHINGTON STATE UNIVERSITY
C               --JAMES J. FILLIBEN
C                 STATISTICAL ENGINEERING DIVISION
C                 CENTER FOR APPLIED MATHEMATICS
C                 NATIONAL BUREAU OF STANDARDS
C
C     LANGUAGE--ANSI FORTRAN (1977)
C     ORIGINAL VERSION--JUNE      1972. 
C     UPDATED         --AUGUST    1974. 
C     UPDATED         --SEPTEMBER 1975. 
C     UPDATED         --NOVEMBER  1975. 
C     UPDATED         --NOVEMBER  1981. 
C     UPDATED         --MAY       1982. 
C     UPDATED         --MARCH     1984. 
C
C-----CHARACTER STATEMENTS FOR NON-COMMON VARIABLES-------------------
C
C---------------------------------------------------------------------
C
      DIMENSION X(*)
C
      DIMENSION M(17)
C
C---------------------------------------------------------------------
C
CCCCC CHARACTER*4 IFEEDB
CCCCC CHARACTER*4 IPRINT
C
CCCCC COMMON /MACH/IRD,IPR,CPUMIN,CPUMAX,NUMBPC,NUMCPW,NUMBPW
CCCCC COMMON /PRINT/IFEEDB,IPRINT
C
C-----SAVE STATEMENTS-------------------------------------------------
C
      SAVE I,J,M,M1,M2
C
C-----DATA STATEMENTS-------------------------------------------------
C
      DATA M(1),M(2),M(3),M(4),M(5),M(6),M(7),M(8),M(9),M(10),M(11),
     1     M(12),M(13),M(14),M(15),M(16),M(17)
     1/    30788,23052,2053,19346,10646,19427,23975,
     1     19049,10949,19693,29746,26748,2796,23890,
     1     29168,31924,16499/ 
      DATA M1,M2,I,J / 32767,256,5,17 / 
C
      IPR=6
C
C-----START POINT-----------------------------------------------------
C
C               ********************************************
C               **  STEP 1--                              **
C               **  CHECK THE INPUT ARGUMENTS FOR ERRORS  **
C               ********************************************
C
      IF(N.GE.1)GOTO90
      WRITE(IPR,999)
  999 FORMAT(1H )
      WRITE(IPR,51) 
   51 FORMAT(1H ,'***** ERROR IN UNIRAN--')
      WRITE(IPR,52) 
   52 FORMAT(1H ,'      THE INPUT NUMBER OF OBSERVATIONS IS ',
     1'NON-POSITIVE.')
      WRITE(IPR,53)N
   53 FORMAT(1H ,'      N = ',I8)
      GOTO9000
   90 CONTINUE
C
C               *******************************************************
C               **  STEP 2--                                         **
C               **  IF A POSITIVE INPUT SEED HAS BEEN GIVEN,         **
C               **  THEN THIS INDICATES THAT THE GENERATOR           **
C               **  SHOULD HAVE ITS INTERNAL M(.) ARRAY REDEFINED--  **
C               **  DO SO IN THIS SECTION.                           **
C               **  IF A NON-POSITIVE INPUT SEED HAS BEEN GIVEN,     **
C               **  THEN THIS INDICATES THAT THE GENERATOR           **
C               **  SHOULD CONTINUE ON FROM WHERE IT LEFT OFF,       **
C               **  AND THEREFORE THIS SECTION IS SKIPPED.           **
C               *******************************************************
C
      IF(ISEED.LE.0)GOTO290
C
CCCCC MDIG=16
      MDIG=32
C
      M1=2**(MDIG-2)+(2**(MDIG-2)-1)
      M2=2**(MDIG/2)
CCCCC ISEED3=MIN0(IABS(ISEED),M1)
      ISEED3=IABS(ISEED)
      IF(M1.LT.IABS(ISEED))ISEED3=M1
      IF(MOD(ISEED3,2).EQ.0)ISEED3=ISEED3-1
      K0=MOD(9069,M2)
      K1=9069/M2
      J0=MOD(ISEED3,M2)
      J1=ISEED3/M2
C
      DO200I=1,17
      ISEED3=J0*K0
      J1=MOD(ISEED3/M2+J0*K1+J1*K0,M2/2)
      J0=MOD(ISEED3,M2)
      M(I)=J0+M2*J1 
  200 CONTINUE
C
      I=5 
      J=17
C
  290 CONTINUE
C
C               *************************************
C               **  STEP 3--                       **
C               **  GENERATE THE N RANDOM NUMBERS  **
C               *************************************
C
      DO300L=1,N
      K=M(I)-M(J)
      IF(K.LT.0)K=K+M1
      M(J)=K
      I=I-1
      IF(I.EQ.0)I=17
      J=J-1
      IF(J.EQ.0)J=17
      AK=K
      AM1=M1
      X(L)=AK/AM1
  300 CONTINUE
C
C               ***************************************************** 
C               **  STEP 4--                                       ** 
C               **  REGARDLESS OF THE VALUE OF THE INPUT SEED,     ** 
C               **  REDEFINE THE VALUE OF ISEED UPON EXIT HERE     ** 
C               **  TO -1 WITH THE NET EFFECT THAT                 ** 
C               **  IF THE USER DOES NOT REDEFINE THE SEED         ** 
C               **  VALUE BEFORE THE NEXT CALL TO THIS GENERATOR,  ** 
C               **  THEN THIS GENERATOR WILL PICK UP               ** 
C               **  WHERE IT LEFT OFF.                             ** 
C               ***************************************************** 
C
      ISEED=(-1)
C
C               *****************
C               **  STEP 90--  **
C               **  EXIT       **
C               *****************
C
 9000 CONTINUE
      RETURN
CCCCC DEBUG TRACE,INIT
CCCCC AT 90
CCCCC TRACE ON
      END 
