package at.boku.survivalkit.skonstructor.form;

import java.awt.EventQueue;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.UIManager;

import at.boku.survivalkit.skonstructor.panels.Prepare1Panel;
import at.boku.survivalkit.skonstructor.panels.Prepare2Panel;
import at.boku.survivalkit.skonstructor.panels.Prepare3Panel;
import at.boku.survivalkit.skonstructor.panels.Weibull1Panel;
import at.boku.survivalkit.skonstructor.panels.Weibull2Panel;
import at.boku.survivalkit.skonstructor.panels.Weibull3Panel;
import at.boku.survivalkit.skonstructor.panels.Weibull4Panel;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

public class ApplicationForm {

	private JFrame frmSkonstructor;
	private AboutFrame aboutFrame;
	
	private JTabbedPane tabbedPane;
	
	private JPanel prepareMainPanel= new JPanel();
	private BorderLayout preparePanelLayout = new BorderLayout(0,0);
	private JPanel weibullMainPanel= new JPanel();
	private BorderLayout weibullPanelLayout = new BorderLayout(0,0);
	
	private Prepare1Panel prepare1Panel;
	private Prepare2Panel prepare2Panel;
	private Prepare3Panel prepare3Panel;
	
	private Weibull1Panel weibull1Panel;
	private Weibull2Panel weibull2Panel;
	private Weibull3Panel weibull3Panel;
	private Weibull4Panel weibull4Panel;
	
	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					
					UIManager.setLookAndFeel(
				            UIManager.getSystemLookAndFeelClassName());

					
					ApplicationForm window = new ApplicationForm();
					window.frmSkonstructor.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Create the application.
	 */
	public ApplicationForm() {
		initialize();
	}

	/**
	 * Initialize the contents of the frame.
	 */
	private void initialize() {
		frmSkonstructor = new JFrame();
		frmSkonstructor.setTitle("SKonstructor");
		frmSkonstructor.setBounds(100, 100, 665, 681);
		frmSkonstructor.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		frmSkonstructor.setJMenuBar(getApplicationJMenuBar());
		
		aboutFrame = new AboutFrame();
		aboutFrame.setLocationRelativeTo(frmSkonstructor);
		
		prepare1Panel = new Prepare1Panel(this);
		prepare2Panel = new Prepare2Panel(this);
		prepare3Panel = new Prepare3Panel(this);
		
		weibull1Panel = new Weibull1Panel(this);
		weibull2Panel = new Weibull2Panel(this);
		weibull3Panel = new Weibull3Panel(this);
		weibull4Panel = new Weibull4Panel(this);
		
		tabbedPane = new JTabbedPane(JTabbedPane.TOP);
		frmSkonstructor.getContentPane().add(tabbedPane, BorderLayout.CENTER);
		tabbedPane.addTab("Prepare", null, prepareMainPanel, null);
		prepareMainPanel.setLayout(preparePanelLayout);
		tabbedPane.addTab("Cox/Weibull", null, weibullMainPanel, null);
		weibullMainPanel.setLayout(weibullPanelLayout);
		
		prepareMainPanel.add(prepare1Panel, BorderLayout.CENTER);
		weibullMainPanel.add(weibull1Panel, BorderLayout.CENTER);
	}

	private JMenuBar getApplicationJMenuBar() {
		JMenuBar jMenuBar = new JMenuBar();
		
		JMenu jMenuFile = new JMenu("File");
		jMenuFile.setMnemonic(KeyEvent.VK_F);
		jMenuBar.add(jMenuFile);
		
		JMenuItem jMenuItemExit = new JMenuItem("Exit");
		jMenuItemExit.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {
				System.exit(0);
			}
			
		});
		jMenuFile.add(jMenuItemExit);
		
		JMenu jMenuHelp = new JMenu("Help");
		jMenuHelp.setMnemonic(KeyEvent.VK_H);
		jMenuBar.add(jMenuHelp);
		
		JMenuItem jMenuItemAbout = new JMenuItem("About");
		jMenuItemAbout.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {
				aboutFrame.setVisible(true);
			}
			
		});
		jMenuHelp.add(jMenuItemAbout);
		
		return jMenuBar;
	}

	public JFrame getFrame() {
		return frmSkonstructor;
	}

	public JPanel getPrepareMainPanel() {
		return prepareMainPanel;
	}

	public BorderLayout getPreparePanelLayout() {
		return preparePanelLayout;
	}

	public JPanel getWeibullMainPanel() {
		return weibullMainPanel;
	}

	public BorderLayout getWeibullPanelLayout() {
		return weibullPanelLayout;
	}

	public Prepare1Panel getPrepare1Panel() {
		return prepare1Panel;
	}

	public Prepare2Panel getPrepare2Panel() {
		return prepare2Panel;
	}

	public Prepare3Panel getPrepare3Panel() {
		return prepare3Panel;
	}

	public Weibull1Panel getWeibull1Panel() {
		return weibull1Panel;
	}

	public Weibull2Panel getWeibull2Panel() {
		return weibull2Panel;
	}

	public Weibull3Panel getWeibull3Panel() {
		return weibull3Panel;
	}

	public Weibull4Panel getWeibull4Panel() {
		return weibull4Panel;
	}

	
	
	
	
}
