package at.boku.survivalkit.skonstructor.panels;

import javax.swing.JPanel;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.LinkedHashMap;
import java.util.Map;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JDesktopPane;
import javax.swing.UIManager;
import javax.swing.JCheckBox;
import javax.swing.JTextField;
import javax.swing.JComboBox;
import javax.swing.JButton;
import javax.swing.JRadioButton;
import javax.swing.ButtonGroup;

import at.boku.survivalkit.skonstructor.form.ApplicationForm;
import javax.swing.JLabel;

public class Weibull3Panel extends JPanel {
	/**
	 * 
	 */
	private static final long serialVersionUID = 7884266151006614140L;
	private JTextField survivalTextField11;
	private JTextField survivalTextField12;
	private JTextField survivalTextField13;
	private JTextField survivalTextField23;
	private JTextField residualTextField1;
	private JTextField residualTextField2;
	private JCheckBox baselineCheckBox;
	private JCheckBox kaplanCheckBox;
	private JComboBox survivalComboBox1;
	private JComboBox survivalComboBox2;
	private JRadioButton survivalRadioButton;
	private JRadioButton residualRadioButton;
	private final ButtonGroup buttonGroup = new ButtonGroup();

	/**
	 * Create the panel.
	 */
	public Weibull3Panel(final ApplicationForm appForm) {
		setLayout(new BorderLayout(0, 0));
		
		JDesktopPane desktopPane = new JDesktopPane();
		desktopPane.setBackground(UIManager.getColor("Button.background"));
		add(desktopPane, BorderLayout.CENTER);
		
		baselineCheckBox = new JCheckBox("baseline");
		baselineCheckBox.setBounds(24, 76, 97, 23);
		desktopPane.add(baselineCheckBox);
		
		kaplanCheckBox = new JCheckBox("kaplan");
		kaplanCheckBox.setBounds(24, 105, 97, 23);
		desktopPane.add(kaplanCheckBox);
		
		survivalTextField11 = new JTextField();
		survivalTextField11.setBounds(93, 134, 91, 19);
		desktopPane.add(survivalTextField11);
		survivalTextField11.setColumns(10);
		
		survivalTextField12 = new JTextField();
		survivalTextField12.setBounds(190, 134, 91, 19);
		desktopPane.add(survivalTextField12);
		survivalTextField12.setColumns(10);
		
		survivalComboBox1 = new JComboBox();
		survivalComboBox1.setBounds(287, 134, 91, 22);
		survivalComboBox1.setModel(new DefaultComboBoxModel(new String[] { "quantiles", "all_times", "equal_space", "specific" }));
		desktopPane.add(survivalComboBox1);
		
		survivalTextField13 = new JTextField();
		survivalTextField13.setBounds(384, 134, 91, 19);
		desktopPane.add(survivalTextField13);
		survivalTextField13.setColumns(10);
		
		survivalComboBox2 = new JComboBox();
		survivalComboBox2.setBounds(287, 162, 91, 22);
		survivalComboBox2.setModel(new DefaultComboBoxModel(new String[] { "", "quantiles", "all_times", "equal_space", "specific" }));
		desktopPane.add(survivalComboBox2);
		
		survivalTextField23 = new JTextField();
		survivalTextField23.setColumns(10);
		survivalTextField23.setBounds(384, 164, 91, 19);
		desktopPane.add(survivalTextField23);
		
		residualTextField1 = new JTextField();
		residualTextField1.setBounds(93, 205, 91, 19);
		desktopPane.add(residualTextField1);
		residualTextField1.setColumns(10);
		
		residualTextField2 = new JTextField();
		residualTextField2.setBounds(190, 205, 91, 19);
		desktopPane.add(residualTextField2);
		residualTextField2.setColumns(10);
		
		JButton weibull3PanelNextButton = new JButton("Next");
		weibull3PanelNextButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				appForm.getWeibullMainPanel().remove(appForm.getWeibull3Panel());
				appForm.getWeibullMainPanel().add(appForm.getWeibull4Panel(), BorderLayout.CENTER);
				appForm.getFrame().repaint();
			}
		});
		weibull3PanelNextButton.setBounds(501, 567, 89, 23);
		desktopPane.add(weibull3PanelNextButton);
		
		JButton weibull3PanelPrevButton = new JButton("Previous");
		weibull3PanelPrevButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				appForm.getWeibullMainPanel().remove(appForm.getWeibull3Panel());
				appForm.getWeibullMainPanel().add(appForm.getWeibull2Panel(), BorderLayout.CENTER);
				appForm.getFrame().repaint();
			}
		});
		weibull3PanelPrevButton.setBounds(406, 567, 89, 23);
		desktopPane.add(weibull3PanelPrevButton);
		
		survivalRadioButton = new JRadioButton("survival");
		buttonGroup.add(survivalRadioButton);
		survivalRadioButton.setBounds(23, 132, 63, 23);
		desktopPane.add(survivalRadioButton);
		
		residualRadioButton = new JRadioButton("residual");
		buttonGroup.add(residualRadioButton);
		residualRadioButton.setBounds(23, 203, 63, 23);
		desktopPane.add(residualRadioButton);
		
		JLabel label = new JLabel("Usage only in Cox");
		label.setBounds(24, 28, 97, 14);
		desktopPane.add(label);

	}
	
	public Map<String, String> getAsMap() {
		Map<String, String> result = new LinkedHashMap<String, String>();
		
		if(baselineCheckBox.isSelected()) {
			result.put(baselineCheckBox.getText(), "");
		}
		
		if(kaplanCheckBox.isSelected()) {
			result.put(kaplanCheckBox.getText(), "");
		}
		if(survivalRadioButton.isSelected()) {
			result.put(survivalRadioButton.getText(), getSurvivalValue());
		}
		if(residualRadioButton.isSelected()) {
			result.put(residualRadioButton.getText(), getResidualValue());
		}
		

		return result;
	}

	private String getResidualValue() {
		StringBuilder sb = new StringBuilder();
		if (residualTextField1.getText() != null && !residualTextField1.getText().equals("")) {
			sb.append(residualTextField1.getText());
		}
		if (residualTextField2.getText() != null && !residualTextField2.getText().equals("")) {
			sb.append(" " + residualTextField2.getText());
		}
		return sb.toString();
	}

	private String getSurvivalValue() {
		StringBuilder sb = new StringBuilder();
		if (survivalTextField11.getText() != null && !survivalTextField11.getText().equals(null)) {
			sb.append(survivalTextField11.getText());
		}
		if (survivalTextField12.getText() != null && !survivalTextField12.getText().equals(null)) {
			sb.append(" " + survivalTextField12.getText());
		}
		sb.append(" " + survivalComboBox1.getSelectedItem());
		if (survivalTextField13.getText() != null && !survivalTextField13.getText().equals(null)) {
			sb.append(" " + survivalTextField13.getText());
		}
		sb.append(" " + survivalComboBox2.getSelectedItem());
		if (survivalTextField23.getText() != null && !survivalTextField23.getText().equals(null)) {
			sb.append(" " + survivalTextField23.getText());
		}
		return sb.toString();
	}
}
