package at.boku.survivalkit.skonstructor.panels;

import javax.swing.JPanel;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.LinkedHashMap;
import java.util.Map;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JDesktopPane;
import javax.swing.UIManager;
import javax.swing.JCheckBox;
import javax.swing.JTextField;
import javax.swing.JComboBox;
import javax.swing.JButton;

import at.boku.survivalkit.skonstructor.form.ApplicationForm;
import at.boku.survivalkit.skonstructor.form.support.ExportAction;
import at.boku.survivalkit.skonstructor.form.support.Utils;

public class Weibull4Panel extends JPanel {
	/**
	 * 
	 */
	private static final long serialVersionUID = -189011741858415969L;
	private JCheckBox strataCheckBox;
	private JCheckBox iteQuasiCheckBox;
	private JCheckBox originCheckBox;
	private JCheckBox rhoFixedCheckBox;
	private JCheckBox coefficientCheckBox;
	private JTextField coefficientTextField1;
	private JTextField coefficientTextField2;
	private JCheckBox animal_solutionCheckBox;
	private JTextField animalSolutionTextField;
	private JCheckBox denseHessianCheckBox;
	private JCheckBox forcePositiveCheckBox;
	private JCheckBox constraintsCheckBox;
	private JComboBox constraintsComboBox;
	private JTextField constraintsTextField;
	private JCheckBox storeSolutionsCheckBox;
	private JCheckBox readOldSolutionsCheckBox;
	private JButton weibull4PanelPrevButton;
	private JButton weibull4PanelCreateParamFileButton;
	private JTextField iteQuasiTextField;
	private JTextField strataTextField;
	private JTextField originTextField;
	private JTextField rhoFixedTextField;

	/**
	 * Create the panel.
	 */
	public Weibull4Panel(final ApplicationForm appForm) {
		setLayout(new BorderLayout(0, 0));
		
		JDesktopPane desktopPane = new JDesktopPane();
		desktopPane.setBackground(UIManager.getColor("Button.background"));
		add(desktopPane, BorderLayout.CENTER);
		
		iteQuasiCheckBox = new JCheckBox("ite_quasi");
		iteQuasiCheckBox.setBounds(10, 48, 85, 23);
		desktopPane.add(iteQuasiCheckBox);
		
		strataCheckBox = new JCheckBox("strata");
		strataCheckBox.setBounds(10, 72, 85, 23);
		desktopPane.add(strataCheckBox);
		
		originCheckBox = new JCheckBox("origin");
		originCheckBox.setBounds(10, 101, 75, 23);
		desktopPane.add(originCheckBox);
		
		rhoFixedCheckBox = new JCheckBox("rho_fixed");
		rhoFixedCheckBox.setBounds(10, 130, 85, 23);
		desktopPane.add(rhoFixedCheckBox);
		
		coefficientCheckBox = new JCheckBox("coefficient");
		coefficientCheckBox.setBounds(10, 159, 75, 23);
		desktopPane.add(coefficientCheckBox);
		
		coefficientTextField1 = new JTextField();
		coefficientTextField1.setBounds(117, 159, 145, 19);
		desktopPane.add(coefficientTextField1);
		coefficientTextField1.setColumns(10);
		
		coefficientTextField2 = new JTextField();
		coefficientTextField2.setBounds(268, 159, 145, 19);
		desktopPane.add(coefficientTextField2);
		coefficientTextField2.setColumns(10);
		
		animal_solutionCheckBox = new JCheckBox("animal_solution");
		animal_solutionCheckBox.setBounds(10, 188, 101, 23);
		desktopPane.add(animal_solutionCheckBox);
		
		animalSolutionTextField = new JTextField();
		animalSolutionTextField.setBounds(117, 190, 145, 19);
		desktopPane.add(animalSolutionTextField);
		animalSolutionTextField.setColumns(10);
		
		denseHessianCheckBox = new JCheckBox("dense_hessian");
		denseHessianCheckBox.setBounds(10, 217, 97, 23);
		desktopPane.add(denseHessianCheckBox);
		
		forcePositiveCheckBox = new JCheckBox("force_positive");
		forcePositiveCheckBox.setBounds(10, 246, 97, 23);
		desktopPane.add(forcePositiveCheckBox);
		
		constraintsCheckBox = new JCheckBox("constraints");
		constraintsCheckBox.setBounds(10, 270, 85, 23);
		desktopPane.add(constraintsCheckBox);
		
		constraintsComboBox = new JComboBox();
		constraintsComboBox.setBounds(117, 270, 106, 22);
		constraintsComboBox.setModel(new DefaultComboBoxModel(new String[] { "largest", "find", "none", "impose", "impose check" }));
		desktopPane.add(constraintsComboBox);	
		
		constraintsTextField = new JTextField();
		constraintsTextField.setBounds(229, 272, 299, 19);
		desktopPane.add(constraintsTextField);
		constraintsTextField.setColumns(10);
		
		storeSolutionsCheckBox = new JCheckBox("store_solutions");
		storeSolutionsCheckBox.setBounds(10, 299, 131, 23);
		desktopPane.add(storeSolutionsCheckBox);
		
		readOldSolutionsCheckBox = new JCheckBox("read_old_solutions");
		readOldSolutionsCheckBox.setBounds(10, 328, 131, 23);
		desktopPane.add(readOldSolutionsCheckBox);
		
		weibull4PanelPrevButton = new JButton("Previous");
		weibull4PanelPrevButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				appForm.getWeibullMainPanel().remove(appForm.getWeibull4Panel());
				appForm.getWeibullMainPanel().add(appForm.getWeibull3Panel(), BorderLayout.CENTER);
				appForm.getFrame().repaint();
			}
		});
		weibull4PanelPrevButton.setBounds(353, 567, 89, 23);
		desktopPane.add(weibull4PanelPrevButton);
		
		weibull4PanelCreateParamFileButton = new JButton("Create parameter file");
		weibull4PanelCreateParamFileButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				
				
					Map<String, String> params = new LinkedHashMap<String, String>();
					params.putAll(appForm.getWeibull1Panel().getAsMap());
					params.putAll(appForm.getWeibull2Panel().getAsMap());
					params.putAll(appForm.getWeibull3Panel().getAsMap());
					params.putAll(getAsMap());
					
					params = Utils.weibullResultTransformer(params);
					
					ExportAction exporter = new ExportAction();
					exporter.exportToFile(appForm.getWeibull4Panel(), params);
			}
		});
		weibull4PanelCreateParamFileButton.setBounds(448, 567, 142, 23);
		desktopPane.add(weibull4PanelCreateParamFileButton);
		
		iteQuasiTextField = new JTextField();
		iteQuasiTextField.setBounds(117, 45, 145, 19);
		desktopPane.add(iteQuasiTextField);
		iteQuasiTextField.setColumns(10);
		
		strataTextField = new JTextField();
		strataTextField.setBounds(117, 72, 145, 19);
		desktopPane.add(strataTextField);
		strataTextField.setColumns(10);
		
		originTextField = new JTextField();
		originTextField.setBounds(117, 101, 145, 19);
		desktopPane.add(originTextField);
		originTextField.setColumns(10);
		
		rhoFixedTextField = new JTextField();
		rhoFixedTextField.setBounds(117, 130, 145, 19);
		desktopPane.add(rhoFixedTextField);
		rhoFixedTextField.setColumns(10);
		
	}
	
	public Map<String, String> getAsMap() {
		Map<String, String> result = new LinkedHashMap<String, String>();
		if(iteQuasiCheckBox.isSelected()) {
			result.put(iteQuasiCheckBox.getText(), iteQuasiTextField.getText());
		}		
		if(strataCheckBox.isSelected()) {
			result.put(strataCheckBox.getText(), strataTextField.getText());
		}
		if(originCheckBox.isSelected()) {
			result.put(originCheckBox.getText(), originTextField.getText());
		}
		if(rhoFixedCheckBox.isSelected()) {
			result.put(rhoFixedCheckBox.getText(), rhoFixedTextField.getText());
		}
		if(coefficientCheckBox.isSelected()) {
			result.put(coefficientCheckBox.getText(), coefficientTextField1.getText() + " " + coefficientTextField2.getText());
		}
		if(animal_solutionCheckBox.isSelected()) {
			result.put(animal_solutionCheckBox.getText(), animalSolutionTextField.getText());
		}
		if(denseHessianCheckBox.isSelected()) {
			result.put(denseHessianCheckBox.getText(), "");
		}
		if(forcePositiveCheckBox.isSelected()) {
			result.put(forcePositiveCheckBox.getText(), "");
		}
		if(constraintsCheckBox.isSelected()) {
			result.put(constraintsCheckBox.getText(), constraintsComboBox.getSelectedItem() + " " + constraintsTextField.getText());
		}
		if(storeSolutionsCheckBox.isSelected()) {
			result.put(storeSolutionsCheckBox.getText(), "");
		}
		if(readOldSolutionsCheckBox.isSelected()) {
			result.put(readOldSolutionsCheckBox.getText(), "");
		}

		return result;
	}
}
